@@ -74,6 +74,12 @@ Dtime::Dtime(QWidget * parent)
   timeServerList->setEnabled(false);
   timeServerList->setEditable(true);
   findNTPutility();
+  if (ntpUtility.isEmpty()) {
+      setDateTimeAuto->setEnabled(false);
+      setDateTimeAuto->setToolTip(i18n("No NTP utility has been found. "
+                                       "Install 'ntpdate' or 'rdate' command to enable automatic "
+                                       "updating of date and time."));
+  }
 
   QVBoxLayout *v2 = new QVBoxLayout( timeBox );
   v2->setMargin( 0 );
@@ -106,10 +112,9 @@ Dtime::Dtime(QWidget * parent)
   connect( cal, SIGNAL(dateChanged(QDate)), SLOT(changeDate(QDate)));
 
   connect( &internalTimer, SIGNAL(timeout()), SLOT(timeout()) );
-  
+
   kclock->setEnabled(false);
-  
-  
+
   //Timezone
   connect( tzonelist, SIGNAL(itemSelectionChanged()), SLOT(configChanged()) );
   tzonesearch->setTreeWidget(tzonelist);
@@ -136,20 +141,24 @@ void Dtime::serverTimeCheck() {
 
 void Dtime::findNTPutility(){
   QByteArray envpath = qgetenv("PATH");
-  if (!envpath.isEmpty() && envpath[0] == ':')
+  if (!envpath.isEmpty() && envpath[0] == ':') {
     envpath = envpath.mid(1);
+  }
+
   QString path = "/sbin:/usr/sbin:";
-  if (!envpath.isEmpty())
+  if (!envpath.isEmpty()) {
     path += QString::fromLocal8Bit(envpath);
-  else
+  } else {
     path += QLatin1String("/bin:/usr/bin");
-  foreach(QString possible_ntputility, QStringList() << "ntpdate" << "rdate" ) {
+  }
+
+  foreach(const QString &possible_ntputility, QStringList() << "ntpdate" << "rdate" ) {
     if( !((ntpUtility = KStandardDirs::findExe(possible_ntputility, path)).isEmpty()) ) {
       kDebug() << "ntpUtility = " << ntpUtility;
       return;
     }
   }
-  ///privateLayoutWidget->hide();
+
   kDebug() << "ntpUtility not found!";
 }
 
@@ -200,7 +209,7 @@ oceania.pool.ntp.org")).split(',', QString::SkipEmptyParts));
   internalTimer.start( 1000 );
 
   timeout();
-  
+
   //Timezone
   currentZone();
 
@@ -222,7 +231,7 @@ void Dtime::save( QVariantMap& helperargs )
     if( list.count() == 10)
       break;
   }
-  
+
   helperargs["ntp"] = true;
   helperargs["ntpServers"] = list;
   helperargs["ntpEnabled"] = setDateTimeAuto->isChecked();
@@ -246,18 +255,17 @@ void Dtime::save( QVariantMap& helperargs )
 
   // restart time
   internalTimer.start( 1000 );
-  
-  //
+
   QStringList selectedZones(tzonelist->selection());
 
-  if (selectedZones.count() > 0){
+  if (selectedZones.count() > 0) {
     QString selectedzone(selectedZones[0]);
     helperargs["tz"] = true;
     helperargs["tzone"] = selectedzone;
   } else {
-      helperargs["tzreset"] = true; // // make the helper reset the timezone
+    helperargs["tzreset"] = true; // // make the helper reset the timezone
   }
-  
+
   currentZone();
 }
 
@@ -272,7 +280,7 @@ void Dtime::processHelperErrors( int code )
   }
   if( code & ClockHelper::TimezoneError)
     KMessageBox::error( this, i18n("Error setting new time zone."),
-        i18n("Time zone Error"));
+                        i18n("Time zone Error"));
 }
 
 void Dtime::timeout()
@@ -336,7 +344,7 @@ void Kclock::setClockSize(const QSize &size)
 }
 
 void Kclock::setTime(const QTime &time)
-{  
+{
     if (time.minute() != this->time.minute() || time.hour() != this->time.hour()) {
         if (m_repaintCache == RepaintNone) {
             m_repaintCache = RepaintHands;
