@@ -28,7 +28,6 @@
 #include <kacceleratormanager.h>
 #include <kapplication.h>
 #include <kglobalsettings.h>
-#include <kgenericfactory.h>
 #include <kmessagebox.h>
 #include <knuminput.h>
 #include <kprocess.h>
@@ -41,6 +40,7 @@
 #include "fonts.moc"
 
 #include <kdebug.h>
+#include <klocale.h>
 
 #ifdef HAVE_FREETYPE
 #include <ft2build.h>
@@ -50,14 +50,18 @@
 #endif
 #endif
 
+#ifdef Q_WS_X11
 #include <X11/Xlib.h>
+#endif
 
 #include <KPluginFactory>
 
+#ifdef Q_WS_X11
 // X11 headers
 #undef Bool
 #undef Unsorted
 #undef None
+#endif
 
 static const char * const aa_rgb_xpm[]={
 "12 12 3 1",
@@ -774,7 +778,7 @@ void KFonts::save()
     KMessageBox::information(this,
       i18n(
         "<p>Some changes such as anti-aliasing will only affect newly started applications.</p>"
-      ), i18n("Font Settings Changed"), "FontSettingsChanged", false);
+      ), i18n("Font Settings Changed"), "FontSettingsChanged");
     useAA_original = useAA;
     dpi_original = dpi;
   }
@@ -783,7 +787,7 @@ void KFonts::save()
     KMessageBox::information(this,
       i18n(
         "<p>Some changes such as DPI will only affect newly started applications.</p>"
-      ), i18n("Font Settings Changed"), "FontSettingsChanged", false);
+      ), i18n("Font Settings Changed"), "FontSettingsChanged");
     dpi_original = dpi;
   }
 #endif
