@@ -248,9 +248,9 @@ void CFontViewPart::timeout()
 
             if(pathList.count()==1)
             {
-                found=KIO::NetAccess::stat("fonts:/"+i18n(KFI_KIO_FONTS_SYS)+'/'+pathList[0], udsEntry, NULL);
+                found=KIO::NetAccess::stat(QString("fonts:/"+i18n(KFI_KIO_FONTS_SYS)+'/'+pathList[0]), udsEntry, NULL);
                 if(!found)
-                    found=KIO::NetAccess::stat("fonts:/"+i18n(KFI_KIO_FONTS_USER)+'/'+pathList[0], udsEntry, NULL);
+                    found=KIO::NetAccess::stat(QString("fonts:/"+i18n(KFI_KIO_FONTS_USER)+'/'+pathList[0]), udsEntry, NULL);
             }
         }
         
@@ -273,7 +273,7 @@ void CFontViewPart::timeout()
     }
     else
     {
-        QString path(url().path());
+        QString path(localFilePath());
 
         // Is this a application/vnd.kde.fontspackage file? If so, extract 1 scalable font...
         if((package=Misc::isPackage(path)))
@@ -333,7 +333,7 @@ void CFontViewPart::timeout()
         FcInitReinitialize();
 
     itsPreview->showFont(!package && itsFontDetails.family.isEmpty()
-                            ? url().path()
+                            ? localFilePath()
                             : fontFile.isEmpty()
                                 ? itsFontDetails.family
                                 : fontFile,
@@ -362,6 +362,8 @@ void CFontViewPart::previewStatus(bool st)
             checkInstallable();
             if(KFI_KIO_FONTS_PROTOCOL==url().protocol())
                 printable=!Misc::isHidden(url());
+            else if(!FC::decode(url()).family.isEmpty())
+                printable=!Misc::isHidden(FC::getFile(url()));
 #ifdef KFI_PRINT_APP_FONTS
             else
             {
