@@ -89,7 +89,7 @@ const uint widgetEventMask =                 // X event mask
 //===========================================================================
 // DLL Interface for kcontrol
 K_PLUGIN_FACTORY(KSSFactory,
-        registerPlugin<KScreenSaver>(); // K_EXPORT_COMPONENT_FACTORY (screensaver
+        registerPlugin<KScreenSaver>();
 )
 K_EXPORT_PLUGIN(KSSFactory("kcmscreensaver"))
 
@@ -210,30 +210,21 @@ KScreenSaver::KScreenSaver(QWidget *parent, const QVariantList&)
 
 //---------------------------------------------------------------------------
 //
-void KScreenSaver::resizeEvent( QResizeEvent * )
+bool KScreenSaver::event(QEvent *e)
 {
-
-  if (mMonitor)
-    {
-      mMonitor->setGeometry( mMonitorPreview->previewRect() );
+    if (e->type() == QEvent::Resize) {
+        if (mMonitor)
+            mMonitor->setGeometry(mMonitorPreview->previewRect());
+    } else if (e->type() == QEvent::KeyPress ||
+               e->type() == QEvent::MouseButtonPress) {
+        if (mTesting) {
+            slotStopTest();
+            return true;
+        }
     }
+    return KCModule::event(e);
 }
 
-//---------------------------------------------------------------------------
-//
-void KScreenSaver::mousePressEvent( QMouseEvent *)
-{
-    if ( mTesting )
-	slotStopTest();
-}
-
-//---------------------------------------------------------------------------
-//
-void KScreenSaver::keyPressEvent( QKeyEvent *)
-{
-    if ( mTesting )
-	slotStopTest();
-}
 //---------------------------------------------------------------------------
 //
 KScreenSaver::~KScreenSaver()
@@ -255,7 +246,7 @@ KScreenSaver::~KScreenSaver()
     {
         if (mSetupProc->state() == QProcess::Running)
         {
-            //Avoid triggering slotPreviewExited on close
+            //Avoid triggering slotSetupDone on close
             mSetupProc->disconnect(this);
 
             mSetupProc->kill( );
