@@ -416,7 +416,8 @@ openGreeter()
     gSet(&grttalk);
     if (grtproc.pid > 0)
         return;
-    if (time(0) < lastStart + 10) /* XXX should use some readiness indicator instead */
+    updateNow();
+    if (now < lastStart + 10) /* XXX should use some readiness indicator instead */
         sessionExit(EX_UNMANAGE_DPY);
     ASPrintf(&name, "greeter for display %s", td->name);
     debug("starting %s\n", name);
@@ -446,7 +447,8 @@ openGreeter()
         sessionExit(EX_UNMANAGE_DPY);
     }
     debug("%s ready\n", name);
-    time(&lastStart);
+    updateNow();
+    lastStart = now;
 }
 
 int
@@ -548,7 +550,7 @@ manageSession(void)
 {
     int ex, cmd;
     volatile int clientPid = -1;
-    volatile time_t tdiff = 0;
+    time_t tdiff;
 
     debug("manageSession %s\n", td->name);
     if ((ex = Setjmp(abortSession))) {
@@ -577,7 +579,8 @@ manageSession(void)
         /* NOTREACHED */
 #endif
 
-    tdiff = time(0) - td->hstent->lastExit - td->openDelay;
+    updateNow();
+    tdiff = now - td->hstent->lastExit - td->openDelay;
     if (autoLogon(tdiff)) {
         if (!verify(conv_auto, False))
             goto gcont;
