@@ -22,7 +22,10 @@
    Boston, MA 02110-1301, USA.
 */
 
+#include <qglobal.h>
+#ifdef Q_WS_X11
 #include <config-X11.h>
+#endif
 
 #include <QtDBus/QDBusConnection>
 
@@ -50,6 +53,10 @@
 #include "historystringitem.h"
 #include "klipperpopup.h"
 
+#ifdef HAVE_DMTX
+#include "mobilebarcode.h"
+#endif
+
 #include <zlib.h>
 
 #ifdef Q_WS_X11
@@ -123,6 +130,7 @@ Klipper::Klipper(QObject *parent, const KSharedConfigPtr &config)
     , m_pendingContentsCheck( false )
     , m_session_managed( new KlipperSessionManager( this ))
 {
+    setenv("KSNI_NO_DBUSMENU", "1", 1);
     QDBusConnection::sessionBus().registerObject("/klipper", this, QDBusConnection::ExportScriptableSlots);
 
     // We don't use the clipboardsynchronizer anymore, and it confuses Klipper
@@ -189,11 +197,6 @@ Klipper::Klipper(QObject *parent, const KSharedConfigPtr &config)
     // if it's too old, drop it and just use load history
     readProperties(m_config.data());
 
-    KAction* a = m_collection->addAction("show_klipper_popup");
-    a->setText(i18n("Show Klipper Popup-Menu"));
-    a->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_V));
-    connect(a, SIGNAL(triggered()), SLOT(slotPopupMenu()));
-
     m_repeatAction = m_collection->addAction("repeat_action");
     m_repeatAction->setText(i18n("Manually Invoke Action on Current Clipboard"));
     qobject_cast<KAction*>(m_repeatAction)->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_R));
@@ -206,13 +209,20 @@ Klipper::Klipper(QObject *parent, const KSharedConfigPtr &config)
     qobject_cast<KAction*>(m_editAction)->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_E), KAction::DefaultShortcut);
     connect(m_editAction, SIGNAL(triggered()), SLOT(slotEditData()));
 
+#ifdef HAVE_DMTX
+    // add barcode for mobile phones
+    m_showBarcodeAction = m_collection->addAction("show_barcode");
+    m_showBarcodeAction->setText(i18n("&Show Barcode..."));
+    connect(m_showBarcodeAction, SIGNAL(triggered()), SLOT(slotShowBarcode()));
+#endif
+
     // Cycle through history
     m_cycleNextAction = m_collection->addAction("cycleNextAction");
-    m_cycleNextAction->setText(i18n("Next history item"));
+    m_cycleNextAction->setText(i18n("Next History Item"));
     m_cycleNextAction->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_Down), KAction::DefaultShortcut);
     connect(m_cycleNextAction, SIGNAL(triggered(bool)), SLOT(slotCycleNext()));
     m_cyclePrevAction = m_collection->addAction("cyclePrevAction");
-    m_cyclePrevAction->setText(i18n("Previous history item"));
+    m_cyclePrevAction->setText(i18n("Previous History Item"));
     m_cyclePrevAction->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_Up), KAction::DefaultShortcut);
     connect(m_cyclePrevAction, SIGNAL(triggered(bool)), SLOT(slotCyclePrev()));
 
@@ -233,6 +243,9 @@ Klipper::Klipper(QObject *parent, const KSharedConfigPtr &config)
     popup->plugAction( m_configureAction );
     popup->plugAction( m_repeatAction );
     popup->plugAction( m_editAction );
+#ifdef HAVE_DMTX
+    popup->plugAction( m_showBarcodeAction );
+#endif
     if ( !isApplet() ) {
         popup->plugAction( m_quitAction );
     }
@@ -306,7 +319,6 @@ void Klipper::loadSettings()
         saveHistory(true);
     }
 
-    m_bPopupAtMouse = KlipperSettings::popupAtMousePosition();
     m_bKeepContents = KlipperSettings::keepClipboardContents();
     m_bReplayActionInHistory = KlipperSettings::replayActionInHistory();
     m_bNoNullClipboard = KlipperSettings::preventEmptyClipboard();
@@ -354,27 +366,14 @@ void Klipper::showPopupMenu( QMenu *menu )
     Q_ASSERT( menu != 0L );
 
     QSize size = menu->sizeHint(); // geometry is not valid until it's shown
-    if (m_bPopupAtMouse) {
-        QPoint g = QCursor::pos();
-        if ( size.height() < g.y() )
-            menu->popup(QPoint( g.x(), g.y() - size.height()));
-        else
-            menu->popup(QPoint(g.x(), g.y()));
-    } else {
-        if( KSystemTrayIcon* tray = dynamic_cast< KSystemTrayIcon* >( parent())) {
-            QRect g = tray->geometry();
-            QRect screen = KGlobalSettings::desktopGeometry(g.center());
+    QPoint pos = QCursor::pos();
+    // ### We can't know where the systray icon is (since it can be hidden or shown 
+    //     in several places), so the cursor position is the only option.
 
-            if ( g.x()-screen.x() > screen.width()/2 &&
-                 g.y()-screen.y() + size.height() > screen.height() )
-                menu->popup(QPoint( g.x(), g.y() - size.height()));
-            else
-                menu->popup(QPoint( g.x() + g.width(), g.y() + g.height()));
-        } else
-            abort();
+    if ( size.height() < pos.y() )
+        pos.ry() -= size.height();
 
-        //      menu->exec(mapToGlobal(QPoint( width()/2, height()/2 )));
-    }
+    menu->popup(pos);
 }
 
 bool Klipper::loadHistory() {
@@ -726,7 +725,7 @@ bool Klipper::blockFetchingNewData()
         return true;
     }
     m_pendingContentsCheck = false;
-    if ( m_overflowCounter == 0 ) 
+    if ( m_overflowCounter == 0 )
         m_overflowClearTimer.start( 1000 );
     if( ++m_overflowCounter > MAX_CLIPBOARD_CHANGES )
         return true;
@@ -792,7 +791,7 @@ void Klipper::checkClipData( bool selectionMode )
 #endif
     const QMimeData* data = m_clip->mimeData( selectionMode ? QClipboard::Selection : QClipboard::Clipboard );
     if ( !data ) {
-        kWarning("No data in clipboard. This not not supposed to happen." );
+        kWarning() << "No data in clipboard. This not not supposed to happen.";
         return;
     }
     // TODO: Rewrite to Qt4 !!!
@@ -1125,11 +1124,33 @@ void Klipper::slotEditData()
 
 }
 
+#ifdef HAVE_DMTX
+void Klipper::slotShowBarcode()
+{
+    const HistoryStringItem* item = dynamic_cast<const HistoryStringItem*>(m_history->first());
+
+    KDialog dlg;
+    dlg.setModal( true );
+    dlg.setCaption( i18n("Mobile Barcode") );
+    dlg.setButtons( KDialog::Ok );
+
+    MobileBarcode::DataMatrixWidget* barcode = new MobileBarcode::DataMatrixWidget( &dlg );
+    if (item) {
+        barcode->setData( item->text() );
+    }
+    barcode->setFocus();
+    dlg.setMainWidget( barcode );
+    dlg.adjustSize();
+
+    dlg.exec();
+}
+#endif //HAVE_DMTX
+
 void Klipper::slotAskClearHistory()
 {
-    int clearHist = KMessageBox::questionYesNo(0, 
+    int clearHist = KMessageBox::questionYesNo(0,
                                                i18n("Really delete entire clipboard history?"),
-                                               i18n("Delete clipboard history?"), 
+                                               i18n("Delete clipboard history?"),
                                                KStandardGuiItem::yes(),
                                                KStandardGuiItem::no(),
                                                QString::fromUtf8("really_clear_history"),
@@ -1185,12 +1206,11 @@ QString Klipper::cycleText() const
 }
 
 // http://bugreports.qt.nokia.com/browse/QTBUG-8157
-#if KDE_IS_VERSION( 4, 4, 85 )
+#if KDE_IS_VERSION( 4, 5, 85 )
 #warning Check if this is still needed, hopefully not.
 #endif
 KlipperEmptyDetector::KlipperEmptyDetector()
 : m_xfixes_event_base( -1 )
-, m_xa_clipboard( None )
 {
 #ifdef HAVE_XFIXES
     m_xa_clipboard = XInternAtom( QX11Info::display(), "CLIPBOARD", False );
@@ -1210,6 +1230,7 @@ KlipperEmptyDetector::KlipperEmptyDetector()
 #endif
 }
 
+#ifdef Q_WS_X11
 bool KlipperEmptyDetector::x11Event( XEvent* e )
 {
 #ifdef HAVE_XFIXES
@@ -1237,5 +1258,6 @@ bool KlipperEmptyDetector::x11Event( XEvent* e )
 #endif
     return false;
 }
+#endif
 
 #include "klipper.moc"
