@@ -176,6 +176,7 @@ void ResultScene::setQueryMatches(const QList<Plasma::QueryMatch> &m)
     QListIterator<ResultItem *> rit(m_items);
     QGraphicsWidget *prevTabItem = 0;
 
+    const int viewableHeight = m_viewableHeight;
     while (mit.hasPrevious() && rit.hasNext()) {
         ResultItem * item = rit.next();
         item->setMatch(mit.previous());
@@ -203,6 +204,9 @@ void ResultScene::setQueryMatches(const QList<Plasma::QueryMatch> &m)
     }
 
     emit matchCountChanged(qMin(m.count(), maxItemsAllowed));
+    if (viewableHeight != m_viewableHeight) {
+        emit viewableHeightChanged();
+    }
 }
 
 void ResultScene::scheduleArrangeItems()
@@ -216,6 +220,7 @@ void ResultScene::arrangeItems()
 {
     int y = 0;
     QListIterator<ResultItem*> matchIt(m_items);
+    const int viewableHeight = m_viewableHeight;
     while (matchIt.hasNext()) {
         ResultItem *item = matchIt.next();
         //kDebug()  << item->name() << item->id() << item->priority() << i;
@@ -229,7 +234,11 @@ void ResultScene::arrangeItems()
     }
 
     //kDebug() << "setting scene rect to" << itemsBoundingRect();
-    setSceneRect(itemsBoundingRect());
+    const QRectF newRect = itemsBoundingRect();
+    setSceneRect(newRect);
+    if (viewableHeight != m_viewableHeight) {
+        emit viewableHeightChanged();
+    }
 }
 
 void ResultScene::highlightItem(QGraphicsItem *item)
@@ -309,25 +318,14 @@ void ResultScene::selectPreviousItem()
     ResultItem *currentFocus = currentlyFocusedItem();
     int currentIndex = currentFocus ? currentFocus->index() : 0;
 
-    bool wrapped = false;
-    if (currentIndex > 0) {
-        currentFocus = m_items.at(currentIndex - 1);
-    } else {
-        currentIndex = m_items.size();
-        do {
-            currentFocus = m_items.at(--currentIndex);
-        } while (currentIndex > 0 && !currentFocus->isVisible());
-        wrapped = currentIndex > 2;
+    if (currentIndex == 0) {
+        m_focusBase->setFocus();
+        return;
     }
 
-    if (currentFocus->isVisible()) {
-        if (wrapped) {
-            // with more than two items, having the selection zoom through the items between looks
-            // odd
-            m_selectionBar->setTargetItem(0);
-            currentFocus->highlight(true);
-        }
+    currentFocus = m_items.at(currentIndex - 1);
 
+    if (currentFocus->isVisible()) {
         setFocusItem(currentFocus);
         emit ensureVisibility(currentFocus);
     }
@@ -337,25 +335,16 @@ void ResultScene::selectNextItem()
 {
     ResultItem *currentFocus = currentlyFocusedItem();
     int currentIndex = currentFocus ? currentFocus->index() : 0;
-    const int wasIndex = currentIndex;
 
-    bool wrapped = false;
     do {
         ++currentIndex;
         if (currentIndex >= m_items.size()) {
-            // with more than two items, having the selection zoom through the items between looks
-            // odd
-            wrapped = wasIndex > 2;
-            currentIndex = 0;
+            return;
         }
         currentFocus = m_items.at(currentIndex);
     } while (!currentFocus->isVisible() && currentIndex < m_items.size());
 
     if (currentFocus->isVisible()) {
-        if (wrapped) {
-            m_selectionBar->setTargetItem(0);
-            currentFocus->highlight(true);
-        }
         setFocusItem(currentFocus);
         emit ensureVisibility(currentFocus);
     }
