@@ -944,7 +944,8 @@ namespace Oxygen
     //________________________________________________________________________________________________________
     TileSet *StyleHelper::holeFocused( const QColor& color, const QColor& glowColor, qreal shade, int size, bool outline )
     {
-        // FIXME must move to s/slabcache/cache/ b/c key is wrong
+
+        // get key
         Oxygen::Cache<TileSet>::Value* cache( m_holeFocusedCache.get( glowColor ) );
 
         const quint64 key( ( quint64( color.rgba() ) << 32 ) | ( quint64( 256.0 * shade ) << 24 ) | size << 1 | outline );
@@ -960,23 +961,42 @@ namespace Oxygen
             p.setRenderHints( QPainter::Antialiasing );
             p.setPen( Qt::NoPen );
 
-            TileSet *holeTileSet = hole( color, shade, size, outline );
-
-            // hole
-            holeTileSet->render( QRect( 0, 0, rsize*2, rsize*2 ), &p );
-
             p.setWindow( 2,2,10,10 );
 
+            // hole mask
+            p.setCompositionMode( QPainter::CompositionMode_DestinationOut );
+            p.setBrush( Qt::black );
+            p.drawEllipse( 3,3,8,8 );
+
+            p.setCompositionMode( QPainter::CompositionMode_SourceOver );
+            if( outline )
+            {
+                QLinearGradient blend( 0, 3, 0, 11 );
+                blend.setColorAt( 0, Qt::transparent );
+                blend.setColorAt( 1, calcDarkColor( color ) );
+
+                p.setBrush( Qt::NoBrush );
+                p.setPen( QPen( blend, 1 ) );
+                p.drawEllipse( 3, 3.5, 8, 7 );
+                p.setPen( Qt::NoPen );
+            }
+
+            // shadow
+            int alpha( glowColor.alpha() );
+            QColor shadowColor( calcShadowColor( color ) );
+            shadowColor.setAlpha( 255-alpha );
+            drawInverseShadow( p, shadowColor, 3, 8, 0.0 );
+
             // glow
             drawInverseGlow( p, glowColor, 3, 8, size );
-
             p.end();
 
             tileSet = new TileSet( pixmap, rsize, rsize, rsize, rsize, rsize-1, rsize, 2, 1 );
-
             cache->insert( key, tileSet );
         }
+
         return tileSet;
+
     }
 
     //______________________________________________________________________________
@@ -1113,16 +1133,15 @@ namespace Oxygen
             p.begin( &pixmap );
             p.setPen( Qt::NoPen );
             p.setRenderHint( QPainter::Antialiasing );
-            QRadialGradient rg = QRadialGradient( 4.5, 4.5, 4.5, 4.5, 4.5 );
+            QRadialGradient rg = QRadialGradient( 4.5, 4.5, 3.5 );
 
             QColor tmpColor( alphaColor( glowColor, 180.0/255 ) );
-            rg.setColorAt( 0.75, tmpColor );
-
+            rg.setColorAt( 1.0, tmpColor );
             tmpColor.setAlpha( 0 );
-            rg.setColorAt( 0.90, tmpColor );
-            rg.setColorAt( 0.4, tmpColor );
+            rg.setColorAt( 0.5, tmpColor );
             p.setBrush( rg );
-            p.drawEllipse( QRectF( 0, 0, 9, 9 ) );
+
+            p.drawEllipse( QRectF( 1, 1, 7, 7 ) );
 
             p.end();
 
