@@ -29,6 +29,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "client.h"
 #include "workspace.h"
 
+#include "scripting/client.h"
+
 #include <fixx11h.h>
 #include <kxerrorhandler.h>
 #include <kstartupinfo.h>
@@ -226,6 +228,12 @@ void Workspace::setActiveClient( Client* c, allowed_t )
     {
     if ( active_client == c )
         return;
+    
+    if(c != 0) {
+	emit clientActivated(c);
+	c->sl_activated();
+    }
+    
     if( active_popup && active_popup_client != c && set_active_client_recursion == 0 ) 
         closeActivePopup();
     StackingUpdatesBlocker blocker( this );
@@ -259,6 +267,9 @@ void Workspace::setActiveClient( Client* c, allowed_t )
     updateColormap();
     if( effects )
         static_cast<EffectsHandlerImpl*>(effects)->windowActivated( active_client ? active_client->effectWindow() : NULL );
+
+    if( tilingEnabled() )
+        notifyTilingWindowActivated( active_client );
     --set_active_client_recursion;
     }
 
@@ -282,12 +293,19 @@ void Workspace::activateClient( Client* c, bool force )
         return;
         }
     raiseClient( c );
-    if (!c->isOnDesktop(currentDesktop()) )
+    if (!c->isOnCurrentDesktop() )
         {
         ++block_focus;
         setCurrentDesktop( c->desktop() );
         --block_focus;
         }
+    if (!c->isOnCurrentActivity() )
+        {
+        ++block_focus;
+        //DBUS!
+        activityController_.setCurrentActivity( c->activities().first() ); //first isn't necessarily best, but it's easiest
+        --block_focus;
+        }
     if( c->isMinimized())
         c->unminimize();
 
@@ -397,10 +415,17 @@ void Workspace::handleTakeActivity( Client* c, Time /*timestamp*/, int flags )
  */
 void Workspace::clientHidden( Client* c )
     {
-    assert( !c->isShown( true ) || !c->isOnCurrentDesktop());
+    assert( !c->isShown( true ) || !c->isOnCurrentDesktop() || !c->isOnCurrentActivity());
     activateNextClient( c );
     }
 
+static inline bool isUsableFocusCandidate( Client *c, Client *prev, bool respectScreen )
+    {
+    return c != prev &&
+           c->isShown( false ) && c->isOnCurrentDesktop() && c->isOnCurrentActivity() &&
+           ( !respectScreen || c->isOnScreen( prev ? prev->screen() : Workspace::self()->activeScreen() ) );
+    }
+
 // deactivates 'c' and activates next client
 bool Workspace::activateNextClient( Client* c )
     {
@@ -419,34 +444,31 @@ bool Workspace::activateNextClient( Client* c )
         {
         if ( options->focusPolicyIsReasonable())
             { // search the focus_chain for a client to transfer focus to,
-              // first try to transfer focus to the first suitable window in the group
             Client* get_focus = NULL;
-            const ClientList windows = ( c != NULL ? c->group()->members() : ClientList());
-	    for ( int i = focus_chain[ currentDesktop() ].size() - 1;
-                  i >= 0;
-                  --i )
+
+            // first try to pass the focus to the (former) active clients leader
+            if ( c  && ( get_focus = c->transientFor() ) &&
+                 isUsableFocusCandidate( get_focus, c, options->separateScreenFocus ))
                 {
-                Client* ci = focus_chain[ currentDesktop() ].at( i );
-                if( c == ci || !ci->isShown( false )
-                    || !ci->isOnCurrentDesktop())
-                    continue;
-                if( options->separateScreenFocus )
-                    {
-                    if( c != NULL && !ci->isOnScreen( c->screen()))
-                        continue;
-                    if( c == NULL && !ci->isOnScreen( activeScreen()))
-                        continue;
-                    }
-                if( windows.contains( ci ))
-                    {
-                    get_focus = ci;
-                    break;
-                    }
-                if( get_focus == NULL )
-                    get_focus = ci;
+                raiseClient( get_focus ); // also raise - we don't know where it came from
                 }
-            if( get_focus == NULL )
+            else // nope, ask the focus chain for the next candidate
+                {
+                get_focus = NULL; // reset
+                for ( int i = focus_chain[ currentDesktop() ].size() - 1; i >= 0; --i )
+                    {
+                    Client* ci = focus_chain[ currentDesktop() ].at( i );
+                    if ( isUsableFocusCandidate( ci, c, options->separateScreenFocus ))
+                        {
+                        get_focus = ci;
+                        break; // we're done
+                        }
+                    }
+                }
+
+            if( get_focus == NULL ) // last chance: focus the desktop
                 get_focus = findDesktop( true, currentDesktop());
+
             if( get_focus != NULL )
                 requestFocus( get_focus );
             else
@@ -475,7 +497,7 @@ void Workspace::setCurrentScreen( int new_screen )
          --i )
         {
         Client* ci = focus_chain[ currentDesktop() ].at( i );
-        if( !ci->isShown( false ) || !ci->isOnCurrentDesktop())
+        if( !ci->isShown( false ) || !ci->isOnCurrentDesktop() || !ci->isOnCurrentActivity())
             continue;
         if( !ci->screen() == new_screen )
             continue;
@@ -761,7 +783,9 @@ Time Client::readUserTimeMapTimestamp( const KStartupInfoId* asn_id, const KStar
     bool session ) const
     {
     Time time = info->userTime();
-    kDebug( 1212 ) << "User timestamp, initial:" << time;
+    //kDebug( 1212 ) << "User timestamp, initial:" << time; 
+    //^^ this deadlocks kwin --replace sometimes.
+
     // newer ASN timestamp always replaces user timestamp, unless user timestamp is 0
     // helps e.g. with konqy reusing
     if( asn_data != NULL && time != 0 )
