@@ -28,39 +28,39 @@
 // IN THE SOFTWARE.
 //////////////////////////////////////////////////////////////////////////////
 
-#include "oxygen.h"
+#include "oxygenanimation.h"
 #include "oxygenclientgroupitemdata.h"
 #include "oxygenconfiguration.h"
+#include "oxygendecohelper.h"
+#include "oxygenfactory.h"
+#include "oxygenshadowcache.h"
 #include "oxygentitleanimationdata.h"
-#include "lib/oxygenanimation.h"
-#include "lib/helper.h"
 
 #include <kcommondecoration.h>
-#include <KGlobalSettings>
 #include <QtCore/QBasicTimer>
+#include <QtCore/QTextStream>
 #include <QtCore/QTimerEvent>
 
 
 namespace Oxygen
 {
 
-    class OxygenSizeGrip;
-    class OxygenClient : public KCommonDecorationUnstable
+    class SizeGrip;
+    class Client : public KCommonDecorationUnstable
     {
 
         Q_OBJECT
 
         //! declare glow intensity property
-        //!Q_PROPERTY( qreal glowIntensity READ glowIntensity WRITE setGlowIntensity )
         Q_PROPERTY( qreal glowIntensity READ glowIntensityUnbiased WRITE setGlowIntensity )
 
         public:
 
         //! constructor
-        OxygenClient(KDecorationBridge *b, OxygenFactory *f);
+        Client(KDecorationBridge *b, Factory *f);
 
         //! destructor
-        virtual ~OxygenClient();
+        virtual ~Client();
 
         //! decoration name
         virtual QString visibleName() const;
@@ -80,17 +80,12 @@ namespace Oxygen
 
 
         //! true if animations are used
-        /*! this combines OxygenConfiguration and KGlobalSettings */
         bool useAnimations( void ) const
-        {
-          return
-            configuration().useAnimations() &&
-            KGlobalSettings::graphicEffectsLevel() >= KGlobalSettings::SimpleAnimationEffects;
-        }
+        { return configuration().useAnimations(); }
 
         //! true if glow is animated
         bool glowIsAnimated( void ) const
-        { return glowAnimation_.data()->isRunning(); }
+        { return glowAnimation_->isRunning(); }
 
         //! true when decoration is forced active
         bool isForcedActive( void ) const
@@ -98,12 +93,30 @@ namespace Oxygen
 
         //! true when separator is to be drawn
         bool drawSeparator( void ) const
+        {
+            if( configuration().drawTitleOutline() ) return false;
+            switch( configuration().separatorMode() )
+            {
+                case Configuration::SeparatorAlways:
+                return true;
+
+                case Configuration::SeparatorActive:
+                return ( glowIsAnimated() || isActive() );
+
+                default:
+                case Configuration::SeparatorNever:
+                return false;
+            }
+
+        }
+
+        //! true if titlebar is hidden
+        bool hideTitleBar( void ) const
         {
             return
-                ( glowIsAnimated() || isActive() ) &&
-                configuration().drawSeparator() &&
-                !configuration().hideTitleBar() &&
-                !configuration().drawTitleOutline();
+                configuration().hideTitleBar() &&
+                !isShade() &&
+                clientGroupItems().count() == 1;
         }
 
         //@}
@@ -118,17 +131,18 @@ namespace Oxygen
         virtual void reset( unsigned long changed );
 
         //! return associated configuration
-        const OxygenConfiguration& configuration( void ) const
+        const Configuration& configuration( void ) const
         { return configuration_; }
 
         //!@name glow animation
         //@{
 
-        virtual const Animation::Pointer& glowAnimation( void ) const
-        { return glowAnimation_; }
-
         void setGlowIntensity( qreal value )
-        { glowIntensity_ = value; }
+        {
+            if( glowIntensity_ == value ) return;
+            glowIntensity_ = value;
+            widget()->update();
+        }
 
         //! unbiased glow intensity
         qreal glowIntensityUnbiased( void ) const
@@ -141,16 +155,16 @@ namespace Oxygen
         //! true (biased) intensity
         /*! this is needed to have glow go from either 0.2->1 or 0.8->0 depending on the animation direction */
         qreal glowIntensity( void ) const
-        { return glowAnimation().data()->direction() == Animation::Forward ? glowIntensity_ : glowIntensity_-glowBias(); }
+        { return glowAnimation_->direction() == Animation::Forward ? glowIntensity_ : glowIntensity_-glowBias(); }
 
         //@}
 
         //! helper class
-        OxygenHelper& helper( void ) const
+        DecoHelper& helper( void ) const
         { return factory_->helper(); }
 
         //! helper class
-        OxygenShadowCache& shadowCache( void ) const
+        ShadowCache& shadowCache( void ) const
         { return factory_->shadowCache(); }
 
         //!@name metrics and color definitions
@@ -210,8 +224,27 @@ namespace Oxygen
         //! resize event
         virtual void resizeEvent(QResizeEvent *e);
 
+        public slots:
+
+        //! triggers widget update in titleRect only
+        /*! one needs to add the title top margin to avoid some clipping glitches */
+        void updateTitleRect( void )
+        { widget()->update( titleRect().adjusted( 0, -layoutMetric( LM_TitleEdgeTop ), 0, 1 ) ); }
+
         protected:
 
+        //! return shadow cache key associated to this client
+        ShadowCache::Key key( void ) const
+        {
+            ShadowCache::Key key;
+            key.active = isActive() || isForcedActive();
+            key.useOxygenShadows = configuration().useOxygenShadows();
+            key.isShade = isShade();
+            key.hasTitleOutline = configuration().drawTitleOutline();
+            key.hasBorder = ( configuration().frameBorder() > Configuration::BorderNone );
+            return key;
+        }
+
         //! true when decoration is forced active
         void setForceActive( bool value )
         { forceActive_ = value; }
@@ -290,7 +323,7 @@ namespace Oxygen
         //@}
 
         //! close tab matching give button
-        virtual bool closeItem( const OxygenButton* );
+        virtual bool closeItem( const Button* );
 
         //! index of item matching point
         int itemClicked( const QPoint& position, bool between = false ) const
@@ -310,18 +343,32 @@ namespace Oxygen
                 useAnimations() &&
                 configuration().animateTitleChange() &&
                 !configuration().drawTitleOutline() &&
-                !configuration().hideTitleBar() &&
+                !hideTitleBar() &&
                 !isPreview();
         }
 
+        //! true if some title outline is rendered
+        bool hasTitleOutline( void ) const
+        {
+            return
+                clientGroupItems().count() >= 2 ||
+                itemData_.isAnimated() ||
+                ( isActive() && configuration().drawTitleOutline() );
+        }
+
         //! calculate mask
         QRegion calcMask( void ) const;
 
         //! text color
-        QColor titlebarTextColor(const QPalette&);
+        QColor titlebarTextColor(const QPalette&) const;
 
         //! text color
-        QColor titlebarTextColor(const QPalette&, bool active);
+        QColor titlebarTextColor(const QPalette& palette, bool active) const
+        {
+            return active ?
+                palette.color(QPalette::Active, QPalette::WindowText):
+                helper().inactiveTitleBarTextColor( palette );
+        }
 
         //! text color
         QColor titlebarContrastColor(const QPalette& palette ) const
@@ -345,7 +392,7 @@ namespace Oxygen
         { return (bool)sizeGrip_; }
 
         //! size grip
-        OxygenSizeGrip& sizeGrip( void ) const
+        SizeGrip& sizeGrip( void ) const
         { return *sizeGrip_; }
 
         //@}
@@ -366,19 +413,19 @@ namespace Oxygen
         private:
 
         //! factory
-        OxygenFactory* factory_;
+        Factory* factory_;
 
         //! size grip widget
-        OxygenSizeGrip* sizeGrip_;
+        SizeGrip* sizeGrip_;
 
         //! configuration
-        OxygenConfiguration configuration_;
+        Configuration configuration_;
 
         //! glow animation
-        Animation::Pointer glowAnimation_;
+        Animation* glowAnimation_;
 
         //! title animation data
-        TitleAnimationData::Pointer titleAnimationData_;
+        TitleAnimationData* titleAnimationData_;
 
         //! glow intensity
         qreal glowIntensity_;
@@ -410,17 +457,6 @@ namespace Oxygen
 
     };
 
-    //!@name utility functions
-    //@{
-
-    // dot
-    void renderDot(QPainter*, const QPointF&, qreal );
-
-    // contrast
-    QColor reduceContrast(const QColor&, const QColor&, double t);
-
-    //@}
-
 } // namespace Oxygen
 
 #endif // EXAMPLECLIENT_H
