@@ -57,6 +57,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <stdio.h>
 
 #include <QMenu>
+#include <QTimerEvent>
 #include <kaction.h>
 #include <kactioncollection.h>
 #include <klocale.h>
@@ -77,6 +78,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 namespace KWin
 {
 
+extern int currentRefreshRate();
+
 //****************************************
 // Workspace
 //****************************************
@@ -195,35 +198,18 @@ void Workspace::setupCompositing()
         delete cm_selection;
         return;
         }
-    int rate = 0;
-    if( options->refreshRate > 0 )
-        { // use manually configured refresh rate
-        rate = options->refreshRate;
+    xrrRefreshRate = KWin::currentRefreshRate();
+    fpsInterval = (options->maxFpsInterval<<10);
+    if ( scene->waitSyncAvailable() ) // if we do vsync, set the fps to the next multiple of the vblank rate
+        {
+        vBlankInterval = (1000<<10)/xrrRefreshRate;
+        fpsInterval -= (fpsInterval % vBlankInterval);
+        fpsInterval = qMax(fpsInterval, vBlankInterval);
         }
-#ifdef HAVE_XRANDR
     else
-        { // autoconfigure refresh rate based on XRandR info
-        if( Extensions::randrAvailable() )
-            {
-            XRRScreenConfiguration *config = XRRGetScreenInfo( display(), rootWindow() );
-            rate = xrrRefreshRate = XRRConfigCurrentRate( config );
-            XRRFreeScreenConfigInfo( config );
-            }
-        }
-#endif
-    // 0Hz or less is invalid, so we fallback to a default rate
-    if( rate <= 0 )
-        rate = 50;
-    // QTimer gives us 1msec (1000Hz) at best, so we ignore anything higher;
-    // however, additional throttling prevents very high rates from taking place anyway
-    else if( rate > 1000 )
-        rate = 1000;
-    kDebug( 1212 ) << "Refresh rate " << rate << "Hz";
-    compositeRate = 1000 / rate;
-    lastCompositePaint.start();
-    // fake a previous paint, so that the next starts right now
-    nextPaintReference = QTime::currentTime().addMSecs( -compositeRate );
-    compositeTimer.setSingleShot( true );
+        vBlankInterval = 1<<10; // no sync - DO NOT set "0", would cause div-by-zero segfaults.
+    vBlankPadding = 3; // vblank rounding errors... :-(
+    nextPaintReference = QTime::currentTime();
     checkCompositeTimer();
     composite_paint_times.clear();
     XCompositeRedirectSubwindows( display(), rootWindow(), CompositeRedirectManual );
@@ -274,7 +260,9 @@ void Workspace::finishCompositing()
     effects = NULL;
     delete scene;
     scene = NULL;
-    compositeTimer.stop();
+    if (compositeTimer)
+        killTimer(compositeTimer);
+    compositeTimer = 0;
     mousePollingTimer.stop();
     repaints_region = QRegion();
     for( ClientList::ConstIterator it = clients.constBegin();
@@ -391,24 +379,25 @@ void Workspace::addRepaintFull()
     checkCompositeTimer();
     }
 
+void Workspace::timerEvent( QTimerEvent *te )
+{
+    if ( te->timerId() == compositeTimer )
+    {
+        killTimer( compositeTimer );
+        compositeTimer = 0;
+        performCompositing();
+    }
+    else
+        QObject::timerEvent( te );
+}
+
 void Workspace::performCompositing()
     {
 #ifdef KWIN_HAVE_COMPOSITING
-    // The event loop apparently tries to fire a QTimer as often as possible, even
-    // at the expense of not processing many X events. This means that the composite
-    // repaints can seriously impact performance of everything else, therefore throttle
-    // them - leave at least 1msec time after one repaint is finished and next one
-    // is started.
-    if( lastCompositePaint.elapsed() < 1 )
-        {
-        compositeTimer.start( 1 );
-        return;
-        }
-    if( !scene->waitSyncAvailable())
-        nextPaintReference = QTime::currentTime();
     if((( repaints_region.isEmpty() && !windowRepaintsPending()) // no damage
         || !overlay_visible )) // nothing is visible anyway
         {
+        vBlankPadding += 3;
         scene->idle();
         // Note: It would seem here we should undo suspended unredirect, but when scenes need
         // it for some reason, e.g. transformations or translucency, the next pass that does not
@@ -424,19 +413,16 @@ void Workspace::performCompositing()
         windows.removeAll( t );
         windows.append( t );
         }
+#if 0
     // skip windows that are not yet ready for being painted
     ToplevelList tmp = windows;
     windows.clear();
-#if 0
     // There is a bug somewhere that prevents this from working properly (#160393), but additionally
     // this cannot be used so carelessly - needs protections against broken clients, the window
     // should not get focus before it's displayed, handle unredirected windows properly and so on.
     foreach( Toplevel* c, tmp )
         if( c->readyForPainting())
             windows.append( c );
-#else
-    foreach( Toplevel* c, tmp )
-        windows.append( c );
 #endif
     foreach( Toplevel* c, windows )
         { // This could be possibly optimized WRT obscuring, but that'd need being already
@@ -444,31 +430,31 @@ void Workspace::performCompositing()
           // TODO I think effects->transformWindowDamage() doesn't need to be called here,
           // pre-paint will extend painted window areas as necessary.
         repaints_region |= c->repaints().translated( c->pos());
-        repaints_region |= c->decorationPendingRegion();        
+        repaints_region |= c->decorationPendingRegion();
         c->resetRepaints( c->decorationRect());
         }
     QRegion repaints = repaints_region;
     // clear all repaints, so that post-pass can add repaints for the next repaint
     repaints_region = QRegion();
     QTime t = QTime::currentTime();
-    scene->paint( repaints, windows );
-    if( scene->waitSyncAvailable())
-        {
-        // If vsync is used, schedule the next repaint slightly in advance of the next sync,
-        // so that there is still time for the drawing to take place. We have just synced, and
-        // nextPaintReference is time from which multiples of compositeRate should be added,
-        // so set it 10ms back (meaning next paint will be in 'compositeRate - 10').
-        // However, make sure the reserve is smaller than the composite rate.
-        int reserve = compositeRate <= 10 ? compositeRate - 1 : 10;
-        nextPaintReference = QTime::currentTime().addMSecs( -reserve );
-        }
+    if ( scene->waitSyncAvailable() )
+    {   // vsync: paint the scene, than rebase the timer and use the duration for next timeout estimation
+        scene->paint( repaints, windows );
+        nextPaintReference = QTime::currentTime();
+    }
+    else
+    {   // no vsyc -> inversion: reset the timer, then paint the scene, this way we can provide a constant framerate
+        nextPaintReference = QTime::currentTime();
+        scene->paint( repaints, windows );
+    }
+    // reset the roundin error corrective... :-(
+    vBlankPadding = 3;
     // Trigger at least one more pass even if there would be nothing to paint, so that scene->idle()
     // is called the next time. If there would be nothing pending, it will not restart the timer and
     // checkCompositeTime() would restart it again somewhen later, called from functions that
     // would again add something pending.
     checkCompositeTimer();
     checkCompositePaintTime( t.elapsed());
-    lastCompositePaint.start();
 #endif
     }
 
@@ -498,9 +484,25 @@ void Workspace::setCompositeTimer()
     {
     if( !compositing()) // should not really happen, but there may be e.g. some damage events still pending
         return;
-    // The last paint set nextPaintReference as a reference time to which multiples of compositeRate
-    // should be added for the next paint. qBound() for protection; system time can change without notice.
-    compositeTimer.start( qBound( 0, nextPaintReference.msecsTo( QTime::currentTime() ), 250 ) % compositeRate );
+    
+    if ( compositeTimer )
+        killTimer( compositeTimer );
+
+    // interval - "time since last paint completion" - "time we need to paint"
+    uint passed = nextPaintReference.msecsTo( QTime::currentTime() ) << 10;
+    uint delay = fpsInterval;
+    if ( scene->waitSyncAvailable() )
+        {
+        if ( passed > fpsInterval )
+            {
+            delay = vBlankInterval;
+            passed %= vBlankInterval;
+            }
+        delay -= ( (passed + ((scene->estimatedRenderTime() + vBlankPadding)<<10) ) % vBlankInterval );
+        }
+    else
+        delay = qBound( 0, int(delay - passed), 250<<10 );
+    compositeTimer = startTimer( delay>>10 );
     }
 
 void Workspace::startMousePolling()
@@ -588,7 +590,9 @@ void Workspace::checkCompositePaintTime( int msec )
                 "If this was only a temporary problem, you can resume using the '%1' shortcut.\n"
                 "You can disable functionality checks in System Settings (on the Advanced tab in Desktop Effects).", shortcut );
         Notify::raise( Notify::CompositingSlow, message );
-        compositeTimer.start( 1000 ); // so that it doesn't trigger sooner than suspendCompositing()
+        if ( compositeTimer )
+            killTimer( compositeTimer );
+        compositeTimer = startTimer( 1000 ); // so that it doesn't trigger sooner than suspendCompositing()
         }
     }
 
@@ -864,6 +868,18 @@ void Toplevel::addDamage( int x, int y, int w, int h )
     damage_region += r;
     repaints_region += r;
     static_cast<EffectsHandlerImpl*>(effects)->windowDamaged( effectWindow(), r );
+    // discard lanczos texture
+    if( effect_window )
+        {
+        QVariant cachedTextureVariant = effect_window->data( LanczosCacheRole );
+        if( cachedTextureVariant.isValid() )
+            {
+            GLTexture *cachedTexture = static_cast< GLTexture*>(cachedTextureVariant.value<void*>());
+            delete cachedTexture;
+            cachedTexture = 0;
+            effect_window->setData( LanczosCacheRole, QVariant() );
+            }
+        }
     workspace()->checkCompositeTimer();
     }
 
@@ -874,6 +890,18 @@ void Toplevel::addDamageFull()
     damage_region = rect();
     repaints_region = rect();
     static_cast<EffectsHandlerImpl*>(effects)->windowDamaged( effectWindow(), rect());
+    // discard lanczos texture
+    if( effect_window )
+        {
+        QVariant cachedTextureVariant = effect_window->data( LanczosCacheRole );
+        if( cachedTextureVariant.isValid() )
+            {
+            GLTexture *cachedTexture = static_cast< GLTexture*>(cachedTextureVariant.value<void*>());
+            delete cachedTexture;
+            cachedTexture = 0;
+            effect_window->setData( LanczosCacheRole, QVariant() );
+            }
+        }
     workspace()->checkCompositeTimer();
     }
 
