@@ -37,6 +37,7 @@ class Deleted
         void unrefWindow( bool delay = false );
         void discard( allowed_t );
         virtual int desktop() const;
+        virtual QStringList activities() const;
         virtual QPoint clientPos() const;
         virtual QSize clientSize() const;
         virtual QRect transparentRect() const;
@@ -49,7 +50,7 @@ class Deleted
         virtual void addRepaintFull(); // in composite.cpp
         QRect decorationRect() const;
     protected:
-        virtual void debug( kdbgstream& stream ) const;
+        virtual void debug( QDebug& stream ) const;
         virtual bool shouldUnredirect() const;
     private:
         Deleted( Workspace *ws ); // use create()
@@ -58,6 +59,7 @@ class Deleted
         int delete_refcount;
         double window_opacity;
         int desk;
+        QStringList activityList;
         QRect contentsRect; // for clientPos()/clientSize()
         QRect transparent_rect;
 
