@@ -22,15 +22,10 @@
 
 #include <X11/Xatom.h>
 
+#include <KDebug>
 
 namespace KWin
 {
-
-enum {
-    BlurRegionRole = 0x1C50A74B 
-};
-
-
 KWIN_EFFECT(blur, BlurEffect)
 KWIN_EFFECT_SUPPORTED(blur, BlurEffect::supported())
 
@@ -50,12 +45,16 @@ BlurEffect::BlurEffect()
     net_wm_blur_region = XInternAtom(display(), "_KDE_NET_WM_BLUR_BEHIND_REGION", False);
     effects->registerPropertyType(net_wm_blur_region, true);
 
+    reconfigure(ReconfigureAll);
+
     // ### Hackish way to announce support.
     //     Should be included in _NET_SUPPORTED instead.
-    XChangeProperty(display(), rootWindow(), net_wm_blur_region, net_wm_blur_region,
-                    32, PropModeReplace, 0, 0);
-
-    reconfigure(ReconfigureAll);
+    if (shader->isValid() && target->valid()) {
+        XChangeProperty(display(), rootWindow(), net_wm_blur_region, net_wm_blur_region,
+                        32, PropModeReplace, 0, 0);
+    } else {
+        XDeleteProperty(display(), rootWindow(), net_wm_blur_region);
+    }
 }
 
 BlurEffect::~BlurEffect()
@@ -75,6 +74,9 @@ void BlurEffect::reconfigure(ReconfigureFlags flags)
     KConfigGroup cg = EffectsHandler::effectConfig("Blur");
     int radius = qBound(2, cg.readEntry("BlurRadius", 12), 14);
     shader->setRadius(radius);
+
+    if (!shader->isValid())
+        XDeleteProperty(display(), rootWindow(), net_wm_blur_region);
 }
 
 void BlurEffect::updateBlurRegion(EffectWindow *w) const
@@ -97,9 +99,9 @@ void BlurEffect::updateBlurRegion(EffectWindow *w) const
         // Set the data to a dummy value.
         // This is needed to be able to distinguish between the value not
         // being set, and being set to an empty region.
-        w->setData(BlurRegionRole, 1);
+        w->setData(WindowBlurBehindRole, 1);
     } else
-        w->setData(BlurRegionRole, region);
+        w->setData(WindowBlurBehindRole, region);
 }
 
 void BlurEffect::windowAdded(EffectWindow *w)
@@ -116,7 +118,16 @@ void BlurEffect::propertyNotify(EffectWindow *w, long atom)
 bool BlurEffect::supported()
 {
     bool supported = GLRenderTarget::supported() && GLTexture::NPOTTextureSupported() &&
-           (GLSLBlurShader::supported() || ARBBlurShader::supported());
+            (GLSLBlurShader::supported() || ARBBlurShader::supported());
+
+    if (supported) {
+        int maxTexSize;
+        glGetIntegerv(GL_MAX_TEXTURE_SIZE, &maxTexSize);
+
+        if (displayWidth() > maxTexSize || displayHeight() > maxTexSize)
+            supported = false;
+    }
+
     if (supported) {
         // check the blacklist
         KSharedConfigPtr config = KSharedConfig::openConfig( "kwinrc" );
@@ -152,11 +163,11 @@ QRegion BlurEffect::blurRegion(const EffectWindow *w) const
 {
     QRegion region;
 
-    const QVariant value = w->data(BlurRegionRole);
+    const QVariant value = w->data(WindowBlurBehindRole);
     if (value.isValid()) {
         const QRegion appRegion = qvariant_cast<QRegion>(value);
         if (!appRegion.isEmpty()) {
-            if (w->hasDecoration()) {
+            if (w->hasDecoration() && effects->decorationSupportsBlurBehind()) {
                 region = w->shape();
                 region -= w->decorationInnerRect();
                 region |= appRegion.translated(w->contentsRect().topLeft()) &
@@ -168,7 +179,7 @@ QRegion BlurEffect::blurRegion(const EffectWindow *w) const
             // for the whole window.
             region = w->shape();
         }
-    } else if (w->hasDecoration()) {
+    } else if (w->hasDecoration() && effects->decorationSupportsBlurBehind()) {
         // If the client hasn't specified a blur region, we'll only enable
         // the effect behind the decoration.
         region = w->shape();
@@ -224,8 +235,8 @@ void BlurEffect::drawWindow(EffectWindow *w, int mask, QRegion region, WindowPai
     const QRect screen(0, 0, displayWidth(), displayHeight());
     bool scaled = !qFuzzyCompare(data.xScale, 1.0) && !qFuzzyCompare(data.yScale, 1.0);
     bool translated = data.xTranslate || data.yTranslate;
-    bool transformed = scaled || translated;
-    bool hasAlpha = w->hasAlpha() || (w->hasDecoration() && effects->decorationsHaveAlpha());
+    bool transformed = scaled || translated || mask & PAINT_WINDOW_TRANSFORMED;
+    bool hasAlpha = w->hasAlpha() || (w->hasDecoration() && effects->decorationsHaveAlpha() && effects->decorationSupportsBlurBehind());
     bool valid = target->valid() && shader->isValid();
 
     QRegion shape;
@@ -236,72 +247,7 @@ void BlurEffect::drawWindow(EffectWindow *w, int mask, QRegion region, WindowPai
 
     if (valid && !shape.isEmpty() && region.intersects(shape.boundingRect()))
     {
-        const QRegion expanded = expand(shape) & screen;
-        const QRect r = expanded.boundingRect();
-
-        // Create a scratch texture and copy the area in the back buffer that we're
-        // going to blur into it
-        GLTexture scratch(r.width(), r.height());
-        scratch.setFilter(GL_LINEAR);
-        scratch.setWrapMode(GL_CLAMP_TO_EDGE);
-        scratch.bind();
-
-        glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, r.x(), displayHeight() - r.y() - r.height(),
-                            r.width(), r.height());
-
-        // Draw the texture on the offscreen framebuffer object, while blurring it horizontally
-        effects->pushRenderTarget(target);
-
-        shader->bind();
-        shader->setDirection(Qt::Horizontal);
-        shader->setPixelDistance(1.0 / r.width());
-
-        // Set up the texture matrix to transform from screen coordinates
-        // to texture coordinates.
-        glMatrixMode(GL_TEXTURE);
-        glPushMatrix();
-        glLoadIdentity();
-        glScalef(1.0 / scratch.width(), -1.0 / scratch.height(), 1);
-        glTranslatef(-r.x(), -scratch.height() - r.y(), 0); 
-
-        drawRegion(expanded);
-
-        effects->popRenderTarget();
-        scratch.unbind();
-        scratch.discard();
-
-        // Now draw the horizontally blurred area back to the backbuffer, while
-        // blurring it vertically and clipping it to the window shape.
-        tex->bind();
-
-        shader->setDirection(Qt::Vertical);
-        shader->setPixelDistance(1.0 / tex->height());
-
-        // Modulate the blurred texture with the window opacity if the window isn't opaque
-        const float opacity = data.opacity * data.contents_opacity;
-        if (opacity < 1.0) {
-            glPushAttrib(GL_COLOR_BUFFER_BIT);
-            glEnable(GL_BLEND);
-            glBlendColor(0, 0, 0, opacity);
-            glBlendFunc(GL_CONSTANT_ALPHA, GL_ONE_MINUS_CONSTANT_ALPHA);
-        }
-
-        // Set the up the texture matrix to transform from screen coordinates
-        // to texture coordinates.
-        glLoadIdentity();
-        glScalef(1.0 / tex->width(), -1.0 / tex->height(), 1);
-        glTranslatef(0, -tex->height(), 0); 
-
-        drawRegion(shape);
-
-        glPopMatrix();
-        glMatrixMode(GL_MODELVIEW);
-
-        if (opacity < 1.0)
-            glPopAttrib();
-
-        tex->unbind(); 
-        shader->unbind(); 
+        doBlur(shape, screen, data.opacity * data.contents_opacity);
 
         // Rebind the shader used for drawing the window if one was set
         if (data.shader)
@@ -312,5 +258,85 @@ void BlurEffect::drawWindow(EffectWindow *w, int mask, QRegion region, WindowPai
     effects->drawWindow(w, mask, region, data);
 }
 
+void BlurEffect::paintEffectFrame(EffectFrame *frame, QRegion region, double opacity, double frameOpacity)
+{
+    const QRect screen(0, 0, displayWidth(), displayHeight());
+    bool valid = target->valid() && shader->isValid();
+    QRegion shape = frame->geometry().adjusted( -5, -5, 5, 5 ) & screen;
+    if (valid && !shape.isEmpty() && region.intersects(shape.boundingRect())) {
+        doBlur(shape, screen, opacity * frameOpacity);
+    }
+    effects->paintEffectFrame(frame, region, opacity, frameOpacity);
+}
+
+void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float opacity)
+{
+    const QRegion expanded = expand(shape) & screen;
+    const QRect r = expanded.boundingRect();
+
+    // Create a scratch texture and copy the area in the back buffer that we're
+    // going to blur into it
+    GLTexture scratch(r.width(), r.height());
+    scratch.setFilter(GL_LINEAR);
+    scratch.setWrapMode(GL_CLAMP_TO_EDGE);
+    scratch.bind();
+
+    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, r.x(), displayHeight() - r.y() - r.height(),
+                        r.width(), r.height());
+
+    // Draw the texture on the offscreen framebuffer object, while blurring it horizontally
+    effects->pushRenderTarget(target);
+
+    shader->bind();
+    shader->setDirection(Qt::Horizontal);
+    shader->setPixelDistance(1.0 / r.width());
+
+    // Set up the texture matrix to transform from screen coordinates
+    // to texture coordinates.
+    glMatrixMode(GL_TEXTURE);
+    glPushMatrix();
+    glLoadIdentity();
+    glScalef(1.0 / scratch.width(), -1.0 / scratch.height(), 1);
+    glTranslatef(-r.x(), -scratch.height() - r.y(), 0);
+
+    drawRegion(expanded);
+
+    effects->popRenderTarget();
+    scratch.unbind();
+    scratch.discard();
+
+    // Now draw the horizontally blurred area back to the backbuffer, while
+    // blurring it vertically and clipping it to the window shape.
+    tex->bind();
+
+    shader->setDirection(Qt::Vertical);
+    shader->setPixelDistance(1.0 / tex->height());
+
+    // Modulate the blurred texture with the window opacity if the window isn't opaque
+    if (opacity < 1.0) {
+        glPushAttrib(GL_COLOR_BUFFER_BIT);
+        glEnable(GL_BLEND);
+        glBlendColor(0, 0, 0, opacity);
+        glBlendFunc(GL_CONSTANT_ALPHA, GL_ONE_MINUS_CONSTANT_ALPHA);
+    }
+
+    // Set the up the texture matrix to transform from screen coordinates
+    // to texture coordinates.
+    glLoadIdentity();
+    glScalef(1.0 / tex->width(), -1.0 / tex->height(), 1);
+    glTranslatef(0, -tex->height(), 0);
+
+    drawRegion(shape);
+
+    glPopMatrix();
+    glMatrixMode(GL_MODELVIEW);
+
+    if (opacity < 1.0)
+        glPopAttrib();
+
+    tex->unbind();
+    shader->unbind();
+}
+
 } // namespace KWin
 
