@@ -1025,6 +1025,8 @@ void CoverSwitchEffect::abort()
 
 void CoverSwitchEffect::windowClosed( EffectWindow* c )
     {
+    if( c == selected_window )
+        selected_window = 0;
     // if the list is not empty, the effect is active
     if( !currentWindowList.isEmpty() )
         {
