@@ -634,6 +634,8 @@ void FlipSwitchEffect::windowAdded( EffectWindow* w )
 
 void FlipSwitchEffect::windowClosed( EffectWindow* w )
     {
+    if( m_selectedWindow == w)
+        m_selectedWindow = 0;
     if( m_active && m_windows.contains( w ) )
         {
         m_windows.remove( w );
