@@ -27,7 +27,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
     #include <kwinglutils.h>
 #endif
 
-
 // This effect shows a preview inside a window that has a special property set
 // on it that says which window and where to render. It is used by the taskbar
 // to show window previews in tooltips.
@@ -76,38 +75,42 @@ void TaskbarThumbnailEffect::paintWindow( EffectWindow* w, int mask, QRegion reg
             mask |= PAINT_WINDOW_OPAQUE;
         else
             mask |= PAINT_WINDOW_TRANSLUCENT;
+        mask |= PAINT_WINDOW_LANCZOS;
         foreach( const Data &thumb, thumbnails.values( w ))
             {
             EffectWindow* thumbw = effects->findWindow( thumb.window );
             if( thumbw == NULL )
                 continue;
             WindowPaintData thumbData( thumbw );
-            thumbData.opacity = data.opacity;
+            thumbData.opacity *= data.opacity;
             QRect r;
-            setPositionTransformations( thumbData, r,
-                thumbw, thumb.rect.translated( w->pos()), Qt::KeepAspectRatio );
 
 #ifdef KWIN_HAVE_OPENGL_COMPOSITING
-            if( effects->compositingType() == KWin::OpenGLCompositing && data.shader )
+            if( effects->compositingType() == KWin::OpenGLCompositing )
                 {
-                // there is a shader - update texture width and height
-                int texw = thumbw->width();
-                int texh = thumbw->height();
-                if( !GLTexture::NPOTTextureSupported() )
+                if ( data.shader )
                     {
-                    kWarning( 1212 ) << "NPOT textures not supported, wasting some memory" ;
-                    texw = nearestPowerOfTwo( texw );
-                    texh = nearestPowerOfTwo( texh );
+                    // there is a shader - update texture width and height
+                    int texw = thumbw->width();
+                    int texh = thumbw->height();
+                    if( !GLTexture::NPOTTextureSupported() )
+                        {
+                        kWarning( 1212 ) << "NPOT textures not supported, wasting some memory" ;
+                        texw = nearestPowerOfTwo( texw );
+                        texh = nearestPowerOfTwo( texh );
+                        }
+                    thumbData.shader = data.shader;
+                    thumbData.shader->setTextureWidth( (float)texw );
+                    thumbData.shader->setTextureHeight( (float)texh );
                     }
-                thumbData.shader = data.shader;
-                thumbData.shader->setTextureWidth( (float)texw );
-                thumbData.shader->setTextureHeight( (float)texh );
-                }
+                } // if ( effects->compositingType() == KWin::OpenGLCompositing ) 
 #endif
+            setPositionTransformations( thumbData, r,
+                    thumbw, thumb.rect.translated( w->pos()), Qt::KeepAspectRatio );
             effects->drawWindow( thumbw, mask, r, thumbData );
             }
         }
-    }
+    } // End of function
 
 void TaskbarThumbnailEffect::windowDamaged( EffectWindow* w, const QRect& damage )
     {
@@ -131,7 +134,7 @@ void TaskbarThumbnailEffect::windowDeleted( EffectWindow* w )
 
 void TaskbarThumbnailEffect::propertyNotify( EffectWindow* w, long a )
     {
-    if( a != atom )
+    if( !w || a != atom )
         return;
     thumbnails.remove( w );
     QByteArray data = w->readProperty( atom, atom, 32 );
