@@ -718,6 +718,10 @@ void KAdvancedConfig::shadeHoverChanged(bool a) {
 
 void KAdvancedConfig::setTilingOn( bool on ) {
     tilingOn->setChecked( on );
+    tilingLayoutLabel->setEnabled( on );
+    tilingLayoutCombo->setEnabled( on );
+    tilingRaiseLabel->setEnabled( on );
+    tilingRaiseCombo->setEnabled( on );
 }
 
 void KAdvancedConfig::setTilingLayout( int l ) {
@@ -1168,6 +1172,8 @@ void KMovingConfig::save( void )
     cg.writeEntry(KWM_CNTR_SNAP_ZONE,getCenterSnapZone());
     cg.writeEntry("SnapOnlyWhenOverlapping",OverlapSnap->isChecked());
 
+    KConfigGroup( config, "Plugins" ).writeEntry("kwin4_effect_windowgeometryEnabled", getGeometryTip());
+
     if (standAlone)
     {
         config->sync();
