@@ -32,6 +32,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QDateTime>
 #include <kmanagerselection.h>
 
+#include "kactivitycontroller.h"
+
 #include "plugins.h"
 #include "utils.h"
 #include "kdecoration.h"
@@ -43,6 +45,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // TODO: Cleanup the order of things in this .h file
 
 class QMenu;
+class QActionGroup;
+class QStringList;
 class KConfig;
 class KActionCollection;
 class KStartupInfo;
@@ -60,6 +64,8 @@ class TabBox;
 }
 
 class Client;
+class Tile;
+class TilingLayout;
 class ClientGroup;
 class DesktopChangeOSD;
 class RootInfo;
@@ -152,6 +158,7 @@ class Workspace : public QObject, public KDecorationDefines
         void raiseClientRequest( Client* c, NET::RequestSource src, Time timestamp );
         void lowerClientRequest( Client* c, NET::RequestSource src, Time timestamp );
         void restackClientUnderActive( Client* );
+        void restack( Client *c, Client *under );
         void updateClientLayer( Client* c );
         void raiseOrLowerClient( Client* );
         void restoreSessionStackingOrder( Client* c );
@@ -168,6 +175,32 @@ class Workspace : public QObject, public KDecorationDefines
         void reserveElectricBorderActions( bool reserve );
         void reserveElectricBorderSwitching( bool reserve );
 
+        //-------------------------------------------------
+        // Tiling
+    public:
+        bool tilingEnabled() const;
+        void setTilingEnabled( bool tiling );
+        bool tileable( Client *c );
+        void createTile( Client *c );
+        // updates geometry of tiles on all desktops,
+        // this rearranges the tiles.
+        void updateAllTiles();
+
+        // The notification functions are called from 
+        // various points in existing code so that
+        // tiling can take any action if required.
+        // They are defined in tiling.cpp
+        void notifyTilingWindowResize( Client *c, const QRect &moveResizeGeom, const QRect &orig );
+        void notifyTilingWindowMove( Client *c, const QRect &moveResizeGeom, const QRect &orig );
+        void notifyTilingWindowResizeDone( Client *c, const QRect &moveResizeGeom, const QRect &orig, bool canceled );
+        void notifyTilingWindowMoveDone( Client *c, const QRect &moveResizeGeom, const QRect &orig, bool canceled );
+        void notifyTilingWindowDesktopChanged( Client *c, int old_desktop );
+        void notifyTilingWindowActivated( Client *c );
+        void notifyTilingWindowMinimizeToggled( Client *c );
+        void notifyTilingWindowMaximized( Client *c, WindowOperation op );
+
+        Position supportedTilingResizeMode( Client *c, Position currentMode );
+
         //-------------------------------------------------
         // Desktop layout
 
@@ -283,8 +316,19 @@ class Workspace : public QObject, public KDecorationDefines
         QSize desktopGridSize_;
         int* desktopGrid_;
         int currentDesktop_;
+        QString activity_;
+        QStringList allActivities_;
         bool desktopLayoutDynamicity_;
 
+        KActivityController activityController_;
+
+        bool tilingEnabled_;
+        // Each tilingLayout is for one virtual desktop.
+        // The length is always one more than the number of
+        // virtual desktops so that we can quickly index them
+        // without having to remember to subtract one.
+        QVector<TilingLayout *> tilingLayouts;
+
         //-------------------------------------------------
         // Unsorted
 
@@ -295,6 +339,9 @@ class Workspace : public QObject, public KDecorationDefines
         void setActiveScreenMouse( const QPoint& mousepos );
         QRect screenGeometry( int screen ) const;
         int screenNumber( const QPoint& pos ) const;
+        QString currentActivity() const { return activity_; }
+        QStringList activityList() const { return allActivities_; }
+        QStringList openActivityList() const { return activityController_.listActivities(KActivityInfo::Running); }
 
         // Tab box
         Client* currentTabBoxClient() const;
@@ -313,7 +360,7 @@ class Workspace : public QObject, public KDecorationDefines
         int previousDesktopStatic( int iDesktop ) const;
         void refTabBox();
         void unrefTabBox();
-        void closeTabBox();
+        void closeTabBox( bool abort = false );
 
         // Tabbing
         void addClientGroup( ClientGroup* group );
@@ -337,6 +384,7 @@ class Workspace : public QObject, public KDecorationDefines
             bool only_normal = true ) const;
         Client* findDesktop( bool topmost, int desktop ) const;
         void sendClientToDesktop( Client* c, int desktop, bool dont_activate );
+        void toggleClientOnActivity( Client* c, const QString &activity, bool dont_activate );
         void windowToPreviousDesktop( Client* c );
         void windowToNextDesktop( Client* c );
         void sendClientToScreen( Client* c, int screen );
@@ -367,11 +415,14 @@ class Workspace : public QObject, public KDecorationDefines
 
         void updateMinimizedOfTransients( Client* );
         void updateOnAllDesktopsOfTransients( Client* );
+        void updateOnAllActivitiesOfTransients( Client* );
         void checkTransients( Window w );
 
         void performWindowOperation( Client* c, WindowOperation op );
 
         void storeSession( KConfig* config, SMSavePhase phase );
+        void storeClient( KConfigGroup &cg, int num, Client *c );
+        void storeSubSession( const QString &name, QSet<QByteArray> sessionIds );
 
         SessionInfo* takeSessionInfo( Client* );
         WindowRules findWindowRules( const Client*, bool );
@@ -384,6 +435,7 @@ class Workspace : public QObject, public KDecorationDefines
         bool decorationHasAlpha() const;
         bool decorationSupportsClientGrouping() const; // Returns true if the decoration supports tabs.
         bool decorationSupportsFrameOverlap() const;
+        bool decorationSupportsBlurBehind() const;
 
         // D-Bus interface
         void cascadeDesktop();
@@ -395,6 +447,11 @@ class Workspace : public QObject, public KDecorationDefines
         void circulateDesktopApplications();
         bool compositingActive();
         bool waitForCompositingSetup();
+        void toggleTiling();
+        void nextTileLayout();
+        void previousTileLayout();
+        bool stopActivity( const QString &id );
+        bool startActivity( const QString &id );
 
         void setCurrentScreen( int new_screen );
 
@@ -651,6 +708,28 @@ class Workspace : public QObject, public KDecorationDefines
         void suspendCompositing();
         void suspendCompositing( bool suspend );
 
+        // user actions, usually bound to shortcuts
+        // and also provided through the D-BUS interface.
+        void slotToggleTiling();
+        void slotToggleFloating();
+        void slotNextTileLayout();
+        void slotPreviousTileLayout();
+
+        // Changes the focused client
+        void slotFocusTileLeft();
+        void slotFocusTileRight();
+        void slotFocusTileTop();
+        void slotFocusTileBottom();
+        // swaps active and adjacent client.
+        void slotMoveTileLeft();
+        void slotMoveTileRight();
+        void slotMoveTileTop();
+        void slotMoveTileBottom();
+        void belowCursor();
+        
+        // NOTE: debug method
+        void dumpTiles() const;
+
         void slotSwitchToTabLeft(); // Slot to move left the active Client.
         void slotSwitchToTabRight(); // Slot to move right the active Client.
         void slotRemoveFromGroup(); // Slot to remove the active client from its group.
@@ -661,8 +740,10 @@ class Workspace : public QObject, public KDecorationDefines
         void slotAddToTabGroup( QAction* ); // Add client to a group
         void slotSwitchToTab( QAction* ); // Change the tab
         void desktopPopupAboutToShow();
+        void activityPopupAboutToShow();
         void clientPopupAboutToShow();
         void slotSendToDesktop( QAction* );
+        void slotToggleOnActivity( QAction* );
         void clientPopupActivated( QAction* );
         void configureWM();
         void desktopResized();
@@ -678,6 +759,7 @@ class Workspace : public QObject, public KDecorationDefines
         void setPopupClientOpacity( QAction* action );
         void setupCompositing();
         void finishCompositing();
+        void fallbackToXRenderCompositing();
         void performCompositing();
         void performMousePoll();
         void lostCMSelection();
@@ -685,17 +767,33 @@ class Workspace : public QObject, public KDecorationDefines
         void resetCursorPosTime();
         void delayedCheckUnredirect();
 
+        void updateCurrentActivity(const QString &new_activity);
+        void activityRemoved(const QString &activity);
+        void activityAdded(const QString &activity);
+        void reallyStopActivity( const QString &id ); //dbus deadlocks suck
+
     protected:
         bool keyPressMouseEmulation( XKeyEvent& ev );
+        void timerEvent( QTimerEvent *te );
 
     Q_SIGNALS:
         Q_SCRIPTABLE void compositingToggled( bool active );
+    
+    //Signals required for the scripting interface
+    signals:
+        void desktopPresenceChanged(KWin::Client*, int);
+	void currentDesktopChanged(int);
+	void clientAdded(KWin::Client*);
+	void clientRemoved(KWin::Client*);
+	void clientActivated(KWin::Client*);
+	void groupAdded(KWin::Group*);
 
     private:
         void init();
         void initShortcuts();
         void readShortcuts();
         void initDesktopPopup();
+        void initActivityPopup();
         void discardPopup();
         void setupWindowShortcut( Client* c );
         void checkCursorPos();
@@ -785,6 +883,9 @@ class Workspace : public QObject, public KDecorationDefines
         Client* active_popup_client;
 
         void loadSessionInfo();
+        void addSessionInfo( KConfigGroup &cg );
+        void loadSubSessionInfo( const QString &name );
+
         void loadWindowRules();
         void editWindowRules( Client* c, bool whole_app );
 
@@ -797,6 +898,16 @@ class Workspace : public QObject, public KDecorationDefines
         static NET::WindowType txtToWindowType( const char* txt );
         static bool sessionInfoWindowTypeMatch( Client* c, SessionInfo* info );
 
+        // try to get a decent tile, either the one with
+        // focus or the one below the mouse.
+        Tile* getNiceTile() const;
+        void removeTile( Client *c );
+        // int, and not Tile::Direction because 
+        // we are using a forward declaration for Tile
+        Tile* findAdjacentTile( Tile *ref, int d );
+        void focusTile( int d );
+        void moveTile( int d );
+
         Client* active_client;
         Client* last_active_client;
         Client* most_recently_raised; // Used ONLY by raiseOrLowerClient()
@@ -856,7 +967,9 @@ class Workspace : public QObject, public KDecorationDefines
         QMenu* popup;
         QMenu* advanced_popup;
         QMenu* trans_popup;
+        QActionGroup* trans_popup_group;
         QMenu* desk_popup;
+        QMenu* activity_popup;
         QMenu* add_tabs_popup; // Menu to add the group to other group
         QMenu* switch_to_tab_popup; // Menu to change tab
 
@@ -868,6 +981,7 @@ class Workspace : public QObject, public KDecorationDefines
         QAction* mMoveOpAction;
         QAction* mMaximizeOpAction;
         QAction* mShadeOpAction;
+        QAction* mTilingStateOpAction;
         QAction* mKeepAboveOpAction;
         QAction* mKeepBelowOpAction;
         QAction* mFullScreenOpAction;
@@ -945,11 +1059,10 @@ class Workspace : public QObject, public KDecorationDefines
 
         KSelectionOwner* cm_selection;
         bool compositingSuspended;
-        QTimer compositeTimer;
-        QTime lastCompositePaint;
+        int compositeTimer;
         QTime nextPaintReference;
         QTimer mousePollingTimer;
-        int compositeRate;
+        uint vBlankInterval, vBlankPadding, fpsInterval, estimatedRenderTime;
         int xrrRefreshRate; // used only for compositing
         QRegion repaints_region;
         Window overlay; // XComposite overlay window
@@ -1088,6 +1201,7 @@ inline Client* Workspace::mostRecentlyActivatedClient() const
 
 inline void Workspace::addGroup( Group* group, allowed_t )
     {
+    emit groupAdded(group);
     groups.append( group );
     }
 
@@ -1132,11 +1246,6 @@ inline void Workspace::sessionSaveStarted()
     session_saving = true;
     }
 
-inline void Workspace::sessionSaveDone()
-    {
-    session_saving = false;
-    }
-
 inline bool Workspace::sessionSaving() const
     {
     return session_saving;
@@ -1239,7 +1348,7 @@ inline bool Workspace::hasClient( const Client* c )
 
 inline void Workspace::checkCompositeTimer()
     {
-    if( !compositeTimer.isActive() )
+    if( !compositeTimer )
         setCompositeTimer();
     }
 
@@ -1263,6 +1372,11 @@ inline bool Workspace::decorationSupportsFrameOverlap() const
     return mgr->factory()->supports( AbilityExtendIntoClientArea );
     }
 
+inline bool Workspace::decorationSupportsBlurBehind() const
+    {
+    return mgr->factory()->supports( AbilityUsesBlurBehind );
+    }
+
 inline void Workspace::addClientGroup( ClientGroup* group )
     {
     clientGroups.append( group );
@@ -1273,6 +1387,30 @@ inline void Workspace::removeClientGroup( ClientGroup* group )
     clientGroups.removeAll( group );
     }
 
+/*
+ * Called from D-BUS
+ */
+inline void Workspace::toggleTiling()
+    {
+    slotToggleTiling();
+    }
+
+/*
+ * Called from D-BUS
+ */
+inline void Workspace::nextTileLayout()
+    {
+    slotNextTileLayout();
+    }
+
+/*
+ * Called from D-BUS
+ */
+inline void Workspace::previousTileLayout()
+    {
+    slotPreviousTileLayout();
+    }
+
 } // namespace
 
 #endif
