@@ -84,7 +84,18 @@ public:
                 QDBusConnection::systemBus()) {}
 };
 
-static enum { Dunno, NoDM, NewKDM, OldKDM, NewGDM, OldGDM } DMType = Dunno;
+class LightDMDBus : public QDBusInterface
+{
+public:
+    LightDMDBus() :
+        QDBusInterface(
+                QLatin1String("org.lightdm.LightDisplayManager"),
+                "/org/lightdm/LightDisplayManager",
+                QLatin1String("org.lightdm.LightDisplayManager"),
+                QDBusConnection::systemBus()) {}
+};
+
+static enum { Dunno, NoDM, NewKDM, OldKDM, NewGDM, OldGDM, LightDM } DMType = Dunno;
 static const char *ctl, *dpy;
 
 class KDisplayManager::Private
@@ -111,8 +122,15 @@ KDisplayManager::KDisplayManager() : d(new Private)
             DMType = NewKDM;
         else if ((ctl = ::getenv("XDM_MANAGED")) && ctl[0] == '/')
             DMType = OldKDM;
-        else if (::getenv("GDMSESSION"))
-            DMType = GDMFactory().isValid() ? NewGDM : OldGDM;
+        else if (::getenv("GDMSESSION")) {
+            //lightDM identifies itself as GDM at the moment.
+            QDBusReply<bool> reply = QDBusConnection::systemBus().interface()->isServiceRegistered("org.lightdm.LightDisplayManager");
+            if (reply.isValid()) {
+                DMType = LightDM;
+            } else {
+                DMType = GDMFactory().isValid() ? NewGDM : OldGDM;
+            }
+        }
         else
             DMType = NoDM;
     }
@@ -283,7 +301,7 @@ static void getSessionLocation(CKSession &lsess, SessEnt &se)
 bool
 KDisplayManager::canShutdown()
 {
-    if (DMType == NewGDM || DMType == NoDM) {
+    if (DMType == NewGDM || DMType == NoDM || DMType == LightDM) {
         QDBusReply<bool> canStop = CKManager().call(QLatin1String("CanStop"));
         return (canStop.isValid() && canStop.value());
     }
@@ -315,7 +333,7 @@ KDisplayManager::shutdown(KWorkSpace::ShutdownType shutdownType,
         if (!bootOption.isEmpty())
             return;
 
-        if (DMType == NewGDM || DMType == NoDM) {
+        if (DMType == NewGDM || DMType == NoDM || DMType == LightDM) {
             // FIXME: entirely ignoring shutdownMode
             CKManager().call(QLatin1String(
                     shutdownType == KWorkSpace::ShutdownTypeReboot ? "Restart" : "Stop"));
@@ -390,7 +408,7 @@ KDisplayManager::setLock(bool on)
 bool
 KDisplayManager::isSwitchable()
 {
-    if (DMType == NewGDM) {
+    if (DMType == NewGDM || DMType == LightDM) {
         QDBusObjectPath currentSeat;
         if (getCurrentSeat(0, &currentSeat)) {
             CKSeat seat(currentSeat);
@@ -417,7 +435,7 @@ KDisplayManager::isSwitchable()
 int
 KDisplayManager::numReserve()
 {
-    if (DMType == NewGDM || DMType == OldGDM)
+    if (DMType == NewGDM || DMType == OldGDM || DMType == LightDM)
         return 1; /* Bleh */
 
     if (DMType == OldKDM)
@@ -438,6 +456,10 @@ KDisplayManager::startReserve()
         GDMFactory().call(QLatin1String("CreateTransientDisplay"));
     else if (DMType == OldGDM)
         exec("FLEXI_XSERVER\n");
+    else if (DMType == LightDM) {
+        LightDMDBus lightDM;
+        lightDM.call("SwitchToUser", QVariant::fromValue<QString>(""));
+    }
     else
         exec("reserve\n");
 }
@@ -448,7 +470,7 @@ KDisplayManager::localSessions(SessList &list)
     if (DMType == OldKDM)
         return false;
 
-    if (DMType == NewGDM) {
+    if (DMType == NewGDM || DMType == LightDM) {
         QDBusObjectPath currentSession, currentSeat;
         if (getCurrentSeat(&currentSession, &currentSeat)) {
             foreach (const QDBusObjectPath &sp, getSessionsForSeat(currentSeat)) {
@@ -546,7 +568,7 @@ KDisplayManager::sess2Str(const SessEnt &se)
 bool
 KDisplayManager::switchVT(int vt)
 {
-    if (DMType == NewGDM) {
+    if (DMType == NewGDM || DMType == LightDM) {
         QDBusObjectPath currentSeat;
         if (getCurrentSeat(0, &currentSeat)) {
             foreach (const QDBusObjectPath &sp, getSessionsForSeat(currentSeat)) {
