@@ -92,6 +92,8 @@ class CalendarTablePrivate
               pendingPopulations(NoPendingPopulation),
               delayedPopulationTimer(new QTimer())
         {
+            KGlobal::locale()->insertCatalog("libkholidays");
+
             svg = new Svg();
             svg->setImagePath("widgets/calendar");
             svg->setContainsMultipleImages(true);
@@ -740,7 +742,7 @@ bool CalendarTable::isAutomaticUpdateEnabled() const
 
 void CalendarTable::setCurrentDate(const QDate &date)
 {
-    d->currentDate = date; 
+    d->currentDate = date;
 }
 
 const QDate& CalendarTable::currentDate() const
@@ -824,7 +826,7 @@ void CalendarTablePrivate::populateCalendar()
 
 void CalendarTablePrivate::populateEvents()
 {
-    pendingPopulations |= PopulateHolidays;
+    pendingPopulations |= PopulateEvents;
     delayedPopulationTimer->start();
 }
 
