@@ -19,96 +19,85 @@
 
 #include "appletsfiltering.h"
 
-#include <kglobalsettings.h>
-#include <klineedit.h>
-#include <kmenu.h>
-#include <kservicetypetrader.h>
-#include <kpushbutton.h>
+#include <QMenu>
 
-#include <plasma/theme.h>
-#include <plasma/corona.h>
-#include <plasma/packagestructure.h>
-#include <plasma/framesvg.h>
-#include <plasma/widgets/label.h>
-#include <plasma/widgets/lineedit.h>
-#include <plasma/widgets/treeview.h>
-#include <plasma/widgets/pushbutton.h>
+#include <KGlobalSettings>
+#include <KLineEdit>
+#include <KMenu>
+#include <KPushButton>
+#include <KServiceTypeTrader>
+#include <KWindowSystem>
+
+#include <Plasma/Theme>
+#include <Plasma/Corona>
+#include <Plasma/PackageStructure>
+#include <Plasma/FrameSvg>
+#include <Plasma/Label>
+#include <Plasma/LineEdit>
+#include <Plasma/PushButton>
+#include <Plasma/ToolButton>
+
+#include <kephal/kephal/screens.h>
 
 #include "widgetexplorer.h"
 #include "openwidgetassistant_p.h"
-//FilteringTreeView
 
-FilteringTreeView::FilteringTreeView(QGraphicsWidget *parent, Qt::WindowFlags wFlags)
-    : Plasma::TreeView(parent)
-{
-    setWindowFlags(wFlags);
-    init();
-    connect(nativeWidget(), SIGNAL(clicked(const QModelIndex &)), this, SLOT(filterChanged(const QModelIndex &)));
-}
+//CategoriesWidget
 
-FilteringTreeView::~FilteringTreeView()
-{
-}
-
-void FilteringTreeView::init()
-{
-    QFont listFont;
-    QPalette plasmaPalette;
-    QColor textColor;
-    QColor color;
-
-    //init treeview
-    nativeWidget()->setAttribute(Qt::WA_NoSystemBackground);
-    nativeWidget()->setEditTriggers(QAbstractItemView::NoEditTriggers);
-    nativeWidget()->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    nativeWidget()->setRootIsDecorated(false);
-    nativeWidget()->setAttribute(Qt::WA_TranslucentBackground);
-
-    //set font and palette
-    listFont = nativeWidget()->font();
-    listFont.setPointSize(KGlobalSettings::smallestReadableFont().pointSize());
-    nativeWidget()->setFont(listFont);
-    plasmaPalette = QPalette();
-    textColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
-    color = Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor);
-    //transparent base color
-    plasmaPalette.setColor(QPalette::Base,
-                           QColor(color.red(), color.green(), color.blue(), 0));
-    plasmaPalette.setColor(QPalette::Text, textColor);
-    setPalette(plasmaPalette);
-    nativeWidget()->setAutoFillBackground(true);
-    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
-
-    nativeWidget()->header()->setVisible(false);
-}
-
-void FilteringTreeView::filterChanged(const QModelIndex & index)
-{
-    emit(filterChanged(index.row()));
-}
-
-//FilteringTabs
-
-FilteringTabs::FilteringTabs(QGraphicsWidget *parent)
-    : Plasma::TabBar(parent),
+CategoriesWidget::CategoriesWidget(QGraphicsWidget *parent)
+    : Plasma::PushButton(parent),
       m_model(0)
 {
-    setAttribute(Qt::WA_NoSystemBackground);
-    nativeWidget()->setUsesScrollButtons(true);
-    connect(this, SIGNAL(currentChanged(int)), this, SIGNAL(filterChanged(int)));
+//    setAttribute(Qt::WA_NoSystemBackground);
+//    nativeWidget()->setUsesScrollButtons(true);
+//    connect(this, SIGNAL(currentChanged(int)), this, SIGNAL(filterChanged(int)));
+    m_menu = new QMenu();
+    m_menu->setTitle(i18n("Categories"));
+    connect(m_menu, SIGNAL(triggered(QAction*)), this, SLOT(menuItemTriggered(QAction*)));
+    setAction(m_menu->menuAction());
 }
 
-FilteringTabs::~FilteringTabs()
+CategoriesWidget::~CategoriesWidget()
 {
-
+    m_menu->deleteLater();
 }
 
-void FilteringTabs::populateList()
+void CategoriesWidget::mousePressEvent(QGraphicsSceneMouseEvent *event)
 {
-    while (count() > 0) {
-        removeTab(0);
+    Plasma::PushButton::mousePressEvent(event);
+    QGraphicsView *view = Plasma::viewFor(this);
+    if (view) {
+        m_menu->adjustSize();
+        QPoint pos = view->mapToGlobal(view->mapFromScene(scenePos()));
+        const QSize msize = m_menu->size();
+        const QRect screen = Kephal::ScreenUtils::screenGeometry(Kephal::ScreenUtils::screenId(view->rect().center()));
+        if (pos.y() - msize.height() > screen.y()) {
+            pos.setY(pos.y() - msize.height());
+        } else {
+            pos = view->mapToGlobal(view->mapFromScene(sceneBoundingRect().bottomLeft()));
+        }
+        m_menu->popup(pos);
+    } else {
+        m_menu->popup(event->screenPos());
     }
+    QTimer::singleShot(0, this, SLOT(unpressButton()));
+}
 
+void CategoriesWidget::unpressButton()
+{
+    nativeWidget()->setDown(false);
+}
+
+void CategoriesWidget::menuItemTriggered(QAction *action)
+{
+    setText(action->text());
+    setIcon(action->icon());
+    emit filterChanged(action->data().toInt());
+}
+
+void CategoriesWidget::populateList()
+{
+    m_menu->clear();
     if (!m_model) {
         return;
     }
@@ -116,18 +105,19 @@ void FilteringTabs::populateList()
     for (int i = 0; i < m_model->rowCount(); i++){
         QStandardItem *item = getItemByProxyIndex(m_model->index(i, 0));
         if (item) {
-            addTab(item->icon(), item->text());
-            nativeWidget()->setTabEnabled(i, item->isEnabled());
+            QAction *action = m_menu->addAction(item->icon(), item->text());
+            action->setData(i);
+            action->setEnabled(item->isEnabled());
         }
     }
 }
 
-QStandardItem *FilteringTabs::getItemByProxyIndex(const QModelIndex &index) const
+QStandardItem *CategoriesWidget::getItemByProxyIndex(const QModelIndex &index) const
 {
     return m_model->itemFromIndex(index);
 }
 
-void FilteringTabs::setModel(QStandardItemModel *model)
+void CategoriesWidget::setModel(QStandardItemModel *model)
 {
     m_model = model;
     populateList();
@@ -138,8 +128,7 @@ void FilteringTabs::setModel(QStandardItemModel *model)
 FilteringWidget::FilteringWidget(QGraphicsItem * parent, Qt::WindowFlags wFlags)
     : QGraphicsWidget(parent, wFlags),
       m_model(0),
-      m_categoriesTreeView(0),
-      m_categoriesTabs(0),
+      m_categories(0),
       m_widgetExplorer(0)
 {
     init();
@@ -152,8 +141,7 @@ FilteringWidget::FilteringWidget(Qt::Orientation orientation,
                                  Qt::WindowFlags wFlags)
     : QGraphicsWidget(parent, wFlags),
       m_model(0),
-      m_categoriesTreeView(0),
-      m_categoriesTabs(0),
+      m_categories(0),
       m_widgetExplorer(widgetExplorer)
 {
     init();
@@ -167,8 +155,6 @@ FilteringWidget::~FilteringWidget()
 
 void FilteringWidget::init()
 {
-    setFocusPolicy(Qt::StrongFocus);
-
     //init text search
     m_textSearch = new Plasma::LineEdit();
 
@@ -184,17 +170,22 @@ void FilteringWidget::init()
     m_newWidgetsButton->nativeWidget()->setMenu(m_newWidgetsMenu);
     m_newWidgetsButton->setMinimumWidth(m_newWidgetsButton->effectiveSizeHint(Qt::PreferredSize).width());
 
+    //close button
+    m_closeButton = new Plasma::ToolButton(this);
+    m_closeButton->setIcon(KIcon("dialog-close"));
+    // Make sure the height of the button is the same as the height of the filtering widget when orientation is Horizontal
+    m_closeButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Minimum);
+    connect(m_closeButton, SIGNAL(clicked()), SIGNAL(closeClicked()));
+
     //layout
-    m_linearLayout = new QGraphicsLinearLayout();
-    m_linearLayout->addItem(m_textSearch);
-    setLayout(m_linearLayout);
+    setListOrientation(m_orientation);
 }
 
 void FilteringWidget::resizeEvent(QGraphicsSceneResizeEvent *event)
 {
     Q_UNUSED(event)
 
-    QSizeF contentsSize = m_linearLayout->contentsRect().size();
+    QSizeF contentsSize = layout()->contentsRect().size();
 //    m_linearLayout->invalidate();
 //    m_linearLayout->activate();
 
@@ -214,78 +205,79 @@ Plasma::LineEdit *FilteringWidget::textSearch()
     return m_textSearch;
 }
 
+void FilteringWidget::updateActions(const QList<QAction *> actions)
+{
+    foreach (QWeakPointer<Plasma::PushButton> button, m_actionButtons) {
+        delete button.data();
+    }
+    m_actionButtons.clear();
+
+    foreach (QAction *action, actions) {
+        Plasma::PushButton *button = new Plasma::PushButton(this);
+        button->setAction(action);
+        int index = qMax(0, layout()->count());
+        if (m_orientation == Qt::Horizontal) {
+            QGraphicsLinearLayout *l = dynamic_cast<QGraphicsLinearLayout*>(layout());
+            l->insertItem(index, button);
+        } else {
+            QGraphicsGridLayout *l = dynamic_cast<QGraphicsGridLayout*>(layout());
+            l->addItem(button, l->rowCount(), 0, 1, 2);
+        }
+        m_actionButtons.append(button);
+    }
+}
+
 void FilteringWidget::setModel(QStandardItemModel *model)
 {
     m_model = model;
-    if (m_categoriesTreeView) {
-        m_categoriesTreeView->setModel(model);
-        int rowHeight = m_categoriesTreeView->nativeWidget()->sizeHintForRow(0);
-        if (rowHeight > 0) {
-            m_categoriesTreeView->setMaximumHeight(rowHeight * 6);
-        }
-    }
 
-    if (m_categoriesTabs) {
-        m_categoriesTabs->setModel(model);
+    if (m_categories) {
+        m_categories->setModel(model);
     }
 }
 
 void FilteringWidget::setListOrientation(Qt::Orientation orientation)
 {
-    if (m_orientation == orientation) {
-        if ((orientation == Qt::Vertical && m_categoriesTreeView) || 
-            (orientation == Qt::Horizontal && m_categoriesTabs)) {
-            return;
-        }
-    }
+    if (m_orientation == orientation && m_categories)
+        return;
 
     m_orientation = orientation;
-    m_linearLayout->setOrientation(orientation);
+
+    if (!m_categories) {
+        m_categories = new CategoriesWidget(this);
+        connect(m_categories, SIGNAL(filterChanged(int)), this, SIGNAL(filterChanged(int)));
+        m_categories->setModel(m_model);
+    }
+
 
     if (orientation == Qt::Horizontal) {
-        if (m_categoriesTreeView) {
-            m_categoriesTreeView->deleteLater();
-            m_categoriesTreeView = 0;
-        }
-
-        if (!m_categoriesTabs) {
-            m_categoriesTabs = new FilteringTabs(this);
-            connect(m_categoriesTabs, SIGNAL(filterChanged(int)), this, SIGNAL(filterChanged(int)));
-            m_categoriesTabs->setModel(m_model);
-        }
-
         m_textSearch->setPreferredWidth(200);
         m_textSearch->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
         m_textSearch->setPreferredHeight(-1);
-        m_linearLayout->addItem(m_categoriesTabs);
-        m_linearLayout->addItem(m_newWidgetsButton);
-        m_categoriesTabs->setVisible(true);
+
+        QGraphicsLinearLayout *newLayout = new QGraphicsLinearLayout(this);
+        newLayout->addItem(m_textSearch);
+        newLayout->addItem(m_categories);
+        newLayout->addStretch();
+        newLayout->addItem(m_newWidgetsButton);
+        newLayout->addItem(m_closeButton);
+        setLayout(newLayout);
+        newLayout->setContentsMargins(4, 0, 0, 4);
     } else {
-        if (m_categoriesTabs) {
-            m_categoriesTabs->deleteLater();
-            m_categoriesTabs = 0;
-        }
-
-        if (!m_categoriesTreeView) {
-            m_categoriesTreeView = new FilteringTreeView(this);
-            connect(m_categoriesTreeView, SIGNAL(filterChanged(int)), this, SIGNAL(filterChanged(int)));
-            m_categoriesTreeView->setModel(m_model);
-            int rowHeight = m_categoriesTreeView->nativeWidget()->sizeHintForRow(0);
-            if (rowHeight > 0) {
-                m_categoriesTreeView->setMaximumHeight(rowHeight * 6);
-            }
-        }
-
         m_textSearch->setPreferredHeight(30);
         m_textSearch->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
         m_textSearch->setPreferredWidth(-1);
-        m_categoriesTreeView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
-        m_linearLayout->addItem(m_categoriesTreeView);
-        m_linearLayout->addItem(m_newWidgetsButton);
-        m_categoriesTreeView->setVisible(true);
+
+        QGraphicsGridLayout *newLayout = new QGraphicsGridLayout(this);
+        newLayout->addItem(m_textSearch, 0, 0);
+        newLayout->addItem(m_closeButton, 0, 1);
+        newLayout->addItem(m_categories, 1, 0, 1, 2);
+        newLayout->addItem(m_newWidgetsButton, 2, 0, 1, 2);
+        setLayout(newLayout);
+        newLayout->setContentsMargins(0, 4, 4, 0);
     }
 
-    m_linearLayout->invalidate();
+    m_categories->setVisible(true);
 }
 
 void FilteringWidget::setMenuPos()
@@ -302,13 +294,6 @@ void FilteringWidget::setMenuPos()
     m_newWidgetsMenu->move(position);
 }
 
-void FilteringWidget::setFocus()
-{
-    QGraphicsWidget::setFocus();
-    m_textSearch->setFocus();
-    m_textSearch->nativeWidget()->setFocus();
- }
-
 void FilteringWidget::populateWidgetsMenu()
 {
     QTimer::singleShot(0, this, SLOT(setMenuPos()));
@@ -316,6 +301,7 @@ void FilteringWidget::populateWidgetsMenu()
         // already populated.
         return;
     }
+
     QSignalMapper *mapper = new QSignalMapper(this);
     QObject::connect(mapper, SIGNAL(mapped(QString)), this, SLOT(downloadWidgets(QString)));
 
@@ -372,6 +358,7 @@ void FilteringWidget::downloadWidgets(const QString &type)
         }
     }
 
+    emit closeClicked();
     if (installer) {
         installer->createNewWidgetBrowser();
     } else {
@@ -383,10 +370,18 @@ void FilteringWidget::downloadWidgets(const QString &type)
 
 void FilteringWidget::openWidgetFile()
 {
-    // TODO: if we already have one of these showing and the user clicks to
-    // add it again, show the same window?
-    Plasma::OpenWidgetAssistant *assistant = new Plasma::OpenWidgetAssistant(0);
+    emit closeClicked();
+
+    Plasma::OpenWidgetAssistant *assistant = m_openAssistant.data();
+    if (!assistant) {
+        assistant = new Plasma::OpenWidgetAssistant(0);
+        m_openAssistant = assistant;
+    }
+
+    KWindowSystem::setOnDesktop(assistant->winId(), KWindowSystem::currentDesktop());
     assistant->setAttribute(Qt::WA_DeleteOnClose, true);
     assistant->show();
+    assistant->raise();
+    assistant->setFocus();
 }
 
