@@ -24,10 +24,12 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "groupmanager.h"
 
 #include <QList>
-#include <KDebug>
+#include <QStack>
 #include <QTimer>
 #include <QUuid>
 
+#include <KDebug>
+
 #include "abstractsortingstrategy.h"
 #include "startup.h"
 #include "task.h"
@@ -40,6 +42,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "strategies/manualgroupingstrategy.h"
 #include "strategies/kustodiangroupingstrategy.h"
 #include "strategies/manualsortingstrategy.h"
+#include "launcheritem.h"
 
 namespace TaskManager
 {
@@ -57,6 +60,7 @@ public:
           currentScreen(-1),
           groupIsFullLimit(0),
           showOnlyCurrentDesktop(false),
+          showOnlyCurrentActivity(false),
           showOnlyCurrentScreen(false),
           showOnlyMinimized(false),
           onlyGroupWhenFull(false),
@@ -72,6 +76,7 @@ public:
     * Keep track of changes in Taskmanager
     */
     void currentDesktopChanged(int);
+    void currentActivityChanged(QString);
     void taskChanged(TaskPtr, ::TaskManager::TaskChanges);
     void checkScreenChange();
     void taskDestroyed(QObject *item);
@@ -82,6 +87,8 @@ public:
     void removeTask(TaskPtr);
     void addStartup(StartupPtr);
     void removeStartup(StartupPtr);
+    void launcherVisibilityChange();
+    void checkLauncherVisibility(LauncherItem *launcher);
 
     TaskGroup *currentRootGroup();
 
@@ -99,19 +106,20 @@ public:
     QSet<Task *> geometryTasks;
     int groupIsFullLimit;
     bool showOnlyCurrentDesktop : 1;
+    bool showOnlyCurrentActivity : 1;
     bool showOnlyCurrentScreen : 1;
     bool showOnlyMinimized : 1;
     bool onlyGroupWhenFull : 1;
     bool changingGroupingStrategy : 1;
     QUuid configToken;
 
-    QHash<int, TaskGroup*> rootGroups; //container for groups
+    QHash<QString, QHash<int, TaskGroup*> > rootGroups; //container for groups
+    QHash<KUrl, LauncherItem *> launchers;
     int currentDesktop;
+    QString currentActivity;
 };
 
 
-
-
 GroupManager::GroupManager(QObject *parent)
     : QObject(parent),
       d(new GroupManagerPrivate(this))
@@ -122,7 +130,9 @@ GroupManager::GroupManager(QObject *parent)
     connect(TaskManager::self(), SIGNAL(startupRemoved(StartupPtr)), this, SLOT(removeStartup(StartupPtr)));
 
     d->currentDesktop = TaskManager::self()->currentDesktop();
-    d->rootGroups[d->currentDesktop] = new TaskGroup(this, "RootGroup", Qt::transparent);
+    d->currentActivity = TaskManager::self()->currentActivity();
+    
+    d->rootGroups[d->currentActivity][d->currentDesktop] = new TaskGroup(this, "RootGroup", Qt::transparent);
 
     d->reloadTimer.setSingleShot(true);
     d->reloadTimer.setInterval(0);
@@ -148,7 +158,7 @@ GroupManager::~GroupManager()
 
 TaskGroup *GroupManagerPrivate::currentRootGroup()
 {
-    return rootGroups[currentDesktop];
+    return rootGroups[currentActivity][currentDesktop];
 }
 
 void GroupManagerPrivate::reloadTasks()
@@ -236,6 +246,12 @@ bool GroupManagerPrivate::addTask(TaskPtr task)
             return false;
         }
 
+        if (showOnlyCurrentActivity && !task->isOnCurrentActivity()) {
+            /* kDebug() << "Not on this desktop and showOnlyCurrentActivity"
+                     << KWindowSystem::currentActivity() << task->desktop(); */
+            return false;
+        }
+
         if (showOnlyMinimized && !task->isMinimized()) {
             //kDebug() << "Not minimized and only showing minimized";
             return false;
@@ -287,6 +303,10 @@ bool GroupManagerPrivate::addTask(TaskPtr task)
 
         QObject::connect(task.data(), SIGNAL(destroyed(QObject*)),
                          q, SLOT(taskDestroyed(QObject*)));
+
+        foreach (LauncherItem *launcher, launchers) {
+            launcher->associateItemIfMatches(item);
+        }
     }
 
     //Find a fitting group for the task with GroupingStrategies
@@ -307,7 +327,6 @@ void GroupManagerPrivate::removeTask(TaskPtr task)
     geometryTasks.remove(task.data());
 
     AbstractGroupableItem *item = currentRootGroup()->getMemberByWId(task->window());
-
     if (!item) {
         // this can happen if the window hasn't been caught previously,
         // of it it is an ignored type such as a NET::Utility type window
@@ -324,7 +343,7 @@ void GroupManagerPrivate::removeTask(TaskPtr task)
 
 void GroupManagerPrivate::taskDestroyed(QObject *item)
 {
-    Task *task = qobject_cast<Task*>(item);
+    Task *task = static_cast<Task*>(item);
     if (task) {
         geometryTasks.remove(task);
     }
@@ -367,6 +386,39 @@ GroupPtr GroupManager::rootGroup() const
     return d->currentRootGroup();
 }
 
+void GroupManagerPrivate::currentActivityChanged(QString newActivity)
+{
+    if (!showOnlyCurrentActivity || currentActivity == newActivity) {
+        return;
+    }
+
+    if (!rootGroups.contains(newActivity) || !rootGroups.value(newActivity).contains(currentDesktop)) {
+        kDebug() << "created new desk group";
+        rootGroups[newActivity][currentDesktop] = new TaskGroup(q, "RootGroup", Qt::transparent);
+        if (abstractSortingStrategy) {
+            abstractSortingStrategy->handleGroup(rootGroups[newActivity][currentDesktop]);
+        }
+    }
+
+    if (onlyGroupWhenFull) {
+        QObject::disconnect(currentRootGroup(), SIGNAL(itemAdded(AbstractGroupableItem *)), q, SLOT(checkIfFull()));
+        QObject::disconnect(currentRootGroup(), SIGNAL(itemRemoved(AbstractGroupableItem *)), q, SLOT(checkIfFull()));
+    }
+
+    currentActivity = newActivity;
+
+    foreach (LauncherItem *item, launchers) {
+        rootGroups[currentActivity][currentDesktop]->add(item);
+    }
+
+    if (onlyGroupWhenFull) {
+        QObject::connect(currentRootGroup(), SIGNAL(itemAdded(AbstractGroupableItem *)), q, SLOT(checkIfFull()));
+        QObject::connect(currentRootGroup(), SIGNAL(itemRemoved(AbstractGroupableItem *)), q, SLOT(checkIfFull()));
+    }
+
+    reloadTasks();
+}
+
 void GroupManagerPrivate::currentDesktopChanged(int newDesktop)
 {
     //kDebug();
@@ -378,26 +430,30 @@ void GroupManagerPrivate::currentDesktopChanged(int newDesktop)
         return;
     }
 
-    if (!rootGroups.contains(newDesktop)) {
+    if (!rootGroups[currentActivity].contains(newDesktop)) {
         kDebug() << "created new desk group";
-        rootGroups[newDesktop] = new TaskGroup(q, "RootGroup", Qt::transparent);
+        rootGroups[currentActivity][newDesktop] = new TaskGroup(q, "RootGroup", Qt::transparent);
         if (abstractSortingStrategy) {
-            abstractSortingStrategy->handleGroup(rootGroups[newDesktop]);
+            abstractSortingStrategy->handleGroup(rootGroups[currentActivity][newDesktop]);
         }
     }
-    
+
     if (onlyGroupWhenFull) {
         QObject::disconnect(currentRootGroup(), SIGNAL(itemAdded(AbstractGroupableItem *)), q, SLOT(checkIfFull()));
         QObject::disconnect(currentRootGroup(), SIGNAL(itemRemoved(AbstractGroupableItem *)), q, SLOT(checkIfFull()));
     }
-    
+
     currentDesktop = newDesktop;
-    
+
+    foreach (LauncherItem *item, launchers) {
+        rootGroups[currentActivity][currentDesktop]->add(item);
+    }
+
     if (onlyGroupWhenFull) {
         QObject::connect(currentRootGroup(), SIGNAL(itemAdded(AbstractGroupableItem *)), q, SLOT(checkIfFull()));
         QObject::connect(currentRootGroup(), SIGNAL(itemRemoved(AbstractGroupableItem *)), q, SLOT(checkIfFull()));
     }
-    
+
     reloadTasks();
 }
 
@@ -414,6 +470,12 @@ void GroupManagerPrivate::taskChanged(TaskPtr task, ::TaskManager::TaskChanges c
         //kDebug() << task->visibleName() << "on" << TaskManager::self()->currentDesktop();
     }
 
+    if (showOnlyCurrentActivity && changes & ::TaskManager::ActivitiesChanged) {
+        takeAction = true;
+        show = task->isOnCurrentActivity();
+        //kDebug() << task->visibleName() << "on" << TaskManager::self()->currentDesktop();
+    }
+
     if (showOnlyMinimized && changes & ::TaskManager::StateChanged) {
         //TODO: wouldn't it be nice to get notification of JUST minimization?
         takeAction = true;
@@ -469,21 +531,26 @@ void GroupManagerPrivate::checkScreenChange()
     }
 }
 
-
 void GroupManager::reconnect()
 {
     kDebug();
     disconnect(TaskManager::self(), SIGNAL(desktopChanged(int)),
                this, SLOT(currentDesktopChanged(int)));
+    disconnect(TaskManager::self(), SIGNAL(activityChanged(QString)),
+               this, SLOT(currentActivityChanged(QString)));
     disconnect(TaskManager::self(), SIGNAL(windowChanged(TaskPtr,::TaskManager::TaskChanges)),
                this, SLOT(taskChanged(TaskPtr,::TaskManager::TaskChanges)));
 
-    if (d->showOnlyCurrentDesktop || d->showOnlyMinimized || d->showOnlyCurrentScreen) {
+    if (d->showOnlyCurrentDesktop || d->showOnlyMinimized || d->showOnlyCurrentScreen || d->showOnlyCurrentActivity) {
         // listen to the relevant task manager signals
         if (d->showOnlyCurrentDesktop) {
             connect(TaskManager::self(), SIGNAL(desktopChanged(int)),
                     this, SLOT(currentDesktopChanged(int)));
         }
+        if (d->showOnlyCurrentActivity) {
+            connect(TaskManager::self(), SIGNAL(activityChanged(QString)),
+                    this, SLOT(currentActivityChanged(QString)));
+        }
 
         connect(TaskManager::self(), SIGNAL(windowChanged(TaskPtr,::TaskManager::TaskChanges)),
                 this, SLOT(taskChanged(TaskPtr,::TaskManager::TaskChanges)));
@@ -497,6 +564,99 @@ void GroupManager::reconnect()
     d->reloadTasks();
 }
 
+bool GroupManager::addLauncher(const KUrl &url, QIcon icon, QString name, QString genericName)
+{
+    if (url.isEmpty()) {
+        return false;
+    }
+
+    LauncherItem *launcher = d->launchers.value(url); // Do not insert launchers twice
+    if (!launcher) {
+        launcher = new LauncherItem(d->currentRootGroup(), url);
+
+        if (!icon.isNull()) {
+            launcher->setIcon(icon);
+        }
+
+        if (!name.isEmpty()) {
+            launcher->setName(name);
+        }
+
+        if (!genericName.isEmpty()) {
+            launcher->setGenericName(genericName);
+        }
+
+        QStack<TaskGroup *> groups;
+        groups.push(d->currentRootGroup());
+        while (!groups.isEmpty()) {
+            TaskGroup *group = groups.pop();
+
+            foreach (AbstractGroupableItem *item, group->members()) {
+                if (item->itemType() == GroupItemType) {
+                    groups.push(static_cast<TaskGroup *>(item));
+                } else {
+                    launcher->associateItemIfMatches(item);
+                }
+            }
+        }
+
+        d->launchers.insert(url, launcher);
+        connect(launcher, SIGNAL(show(bool)), this, SLOT(launcherVisibilityChange()));
+        d->checkLauncherVisibility(launcher);
+        emit launcherAdded(launcher);
+    }
+
+    return launcher;
+}
+
+void GroupManager::removeLauncher(const KUrl &url)
+{
+    LauncherItem *launcher = d->launchers.value(url);
+    if (!launcher) {
+        return;
+    }
+
+    d->launchers.remove(url);
+
+    typedef QHash<int,TaskGroup*> Metagroup;
+    foreach (Metagroup metagroup, d->rootGroups) {
+        foreach (TaskGroup *rootGroup, metagroup) {
+            rootGroup->remove(launcher);
+        }
+    }
+
+    emit launcherRemoved(launcher);
+    launcher->deleteLater();
+}
+
+void GroupManagerPrivate::launcherVisibilityChange()
+{
+    checkLauncherVisibility(qobject_cast<LauncherItem *>(q->sender()));
+}
+
+void GroupManagerPrivate::checkLauncherVisibility(LauncherItem *launcher)
+{
+    kDebug() << "booyah, bitches!" << launcher;
+    if (!launcher) {
+        return;
+    }
+
+    typedef QHash<int, TaskGroup *> Metagroup;
+    foreach (Metagroup metagroup, rootGroups) {
+        foreach (TaskGroup *rootGroup, metagroup) {
+            if (launcher->shouldShow()) {
+                rootGroup->add(launcher);
+            } else {
+                rootGroup->remove(launcher);
+            }
+        }
+    }
+}
+
+bool GroupManager::launcherExists(const KUrl &url) const
+{
+    return d->launchers.value(url);
+}
 
 bool GroupManager::onlyGroupWhenFull() const
 {
@@ -583,6 +743,17 @@ void GroupManager::setShowOnlyCurrentDesktop(bool showOnlyCurrentDesktop)
     reconnect();
 }
 
+bool GroupManager::showOnlyCurrentActivity() const
+{
+    return d->showOnlyCurrentActivity;
+}
+
+void GroupManager::setShowOnlyCurrentActivity(bool showOnlyCurrentActivity)
+{
+    d->showOnlyCurrentActivity = showOnlyCurrentActivity;
+    reconnect();
+}
+
 bool GroupManager::showOnlyMinimized() const
 {
     return d->showOnlyMinimized;
@@ -637,8 +808,11 @@ void GroupManager::setSortingStrategy(TaskSortingStrategy sortOrder)
             kDebug() << "Invalid Strategy";
     }
     if (d->abstractSortingStrategy) {
-        foreach (TaskGroup *group, d->rootGroups) {
-            d->abstractSortingStrategy->handleGroup(group);
+        typedef QHash<int,TaskGroup*> Metagroup;
+        foreach (Metagroup metagroup, d->rootGroups) {
+            foreach (TaskGroup *group, metagroup) {
+                d->abstractSortingStrategy->handleGroup(group);
+            }
         }
     }
 
@@ -713,4 +887,3 @@ void GroupManager::setGroupingStrategy(TaskGroupingStrategy strategy)
 } // TaskManager namespace
 
 #include "groupmanager.moc"
-
