@@ -36,12 +36,16 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <kicon.h>
 #include <klocale.h>
 #include <KDebug>
+#include <KService>
+#include <KServiceTypeTrader>
+#include <KStandardDirs>
+#include <KDesktopFile>
 
 namespace TaskManager
 {
 
 
-QAction *standardGroupableAction(GroupableAction action, AbstractGroupableItem *item, QObject *parent, int desktop)
+QAction *standardGroupableAction(GroupableAction action, AbstractGroupableItem *item, GroupManager *strategy, QObject *parent, int desktop)
 {
     Q_ASSERT(item);
 
@@ -70,6 +74,9 @@ QAction *standardGroupableAction(GroupableAction action, AbstractGroupableItem *
         case KeepBelowAction:
             return new KeepBelowActionImpl(parent, item);
             break;
+        case ToggleLauncherAction:
+            return new ToggleLauncherActionImpl(parent, item, strategy);
+            break;
     }
 
     return 0;
@@ -165,26 +172,50 @@ CloseActionImpl::CloseActionImpl(QObject *parent, AbstractGroupableItem *item)
 }
 
 
+AbstractGroupableItemAction::AbstractGroupableItemAction(QObject *parent, AbstractGroupableItem *item)
+    : QAction(parent)
+{
+    TaskGroup *group = qobject_cast<TaskGroup *>(item);
+    if (group) {
+        addToTasks(group);
+    } else if (TaskItem *taskItem = qobject_cast<TaskItem *>(item)) {
+        m_tasks.append(taskItem->task());
+    } 
+}
+
+void AbstractGroupableItemAction::addToTasks(TaskGroup *group)
+{
+    foreach (AbstractGroupableItem *item, group->members()) {
+        TaskGroup *subGroup = qobject_cast<TaskGroup *>(item);
+        if (subGroup) {
+            addToTasks(subGroup);
+        } else if (TaskItem *taskItem = qobject_cast<TaskItem *>(item)) {
+            m_tasks.append(taskItem->task());
+        }
+    }
+}
+
 ToCurrentDesktopActionImpl::ToCurrentDesktopActionImpl(QObject *parent, AbstractGroupableItem *item)
-    : QAction(parent),
-      m_item(item)
+    : AbstractGroupableItemAction(parent, item)
 {
     connect(this, SIGNAL(triggered()), this, SLOT(slotToCurrentDesktop()));
-    setText(i18n("&To Current Desktop"));
+    setText(i18n("Move &To Current Desktop"));
     setEnabled(!item->isOnCurrentDesktop() && item->isActionSupported(NET::ActionChangeDesktop));
 }
 
 void ToCurrentDesktopActionImpl::slotToCurrentDesktop() 
 {
-    m_item->toDesktop(KWindowSystem::currentDesktop());
+    const int desktop = KWindowSystem::currentDesktop();
+    foreach (TaskPtr task, m_tasks) {
+        task->toDesktop(desktop);
+    }
 }
 
 
 
 ToDesktopActionImpl::ToDesktopActionImpl(QObject *parent, AbstractGroupableItem *item, int desktop)
-    : QAction(parent),
-      m_desktop(desktop),
-      m_item(item)
+    : AbstractGroupableItemAction(parent, item),
+      m_desktop(desktop)
 {
     connect(this, SIGNAL(triggered()), this, SLOT(slotToDesktop()));
     setCheckable(true);
@@ -201,7 +232,9 @@ ToDesktopActionImpl::ToDesktopActionImpl(QObject *parent, AbstractGroupableItem
 
 void ToDesktopActionImpl::slotToDesktop() 
 {
-    m_item->toDesktop(m_desktop);
+    foreach (TaskPtr task, m_tasks) {
+        task->toDesktop(m_desktop);
+    }
 }
 
 
@@ -209,7 +242,7 @@ void ToDesktopActionImpl::slotToDesktop()
 DesktopsMenu::DesktopsMenu(QWidget *parent, AbstractGroupableItem *item)
     : QMenu(parent)
 {
-    setTitle( i18n("To &Desktop") );
+    setTitle( i18n("Move To &Desktop") );
     addAction( new ToDesktopActionImpl(this,item,0) );      //0 means all desktops
     addSeparator();
     for (int i = 1; i <= TaskManager::self()->numberOfDesktops(); i++) {
@@ -249,13 +282,26 @@ ViewFullscreenActionImpl::ViewFullscreenActionImpl(QObject *parent, AbstractGrou
     setEnabled(item->isActionSupported(NET::ActionFullScreen));
 }
 
-AdvancedMenu::AdvancedMenu(QWidget *parent, AbstractGroupableItem *item)
-    :QMenu(parent)
+AdvancedMenu::AdvancedMenu(QWidget *parent, AbstractGroupableItem *item, GroupManager *strategy)
+    : QMenu(parent)
 {
     setTitle(i18n("Ad&vanced"));
     addAction(new KeepAboveActionImpl(this, item));
     addAction(new KeepBelowActionImpl(this, item));
     addAction(new ViewFullscreenActionImpl(this, item));
+
+    addSeparator();
+
+    addAction(new ToggleLauncherActionImpl(this, item, strategy));
+    if (strategy->taskGrouper()) {
+        QList<QAction*> groupingStrategyActions = strategy->taskGrouper()->strategyActions(this, item);
+        if (!groupingStrategyActions.isEmpty()) {
+            foreach (QAction *action, groupingStrategyActions) {
+                addAction(action);
+            }
+            // delete groupingStrategyActions;
+        }
+    }
 }
 
 LeaveGroupActionImpl::LeaveGroupActionImpl(QObject *parent, AbstractGroupableItem *item, GroupManager *strategy)
@@ -273,6 +319,63 @@ void LeaveGroupActionImpl::leaveGroup()
     groupingStrategy->manualGroupingRequest(abstractItem,abstractItem->parentGroup()->parentGroup());
 }
 
+ToggleLauncherActionImpl::ToggleLauncherActionImpl(QObject *parent, AbstractGroupableItem *item, GroupManager *strategy)
+    : QAction(parent), m_abstractItem(item), m_groupingStrategy(strategy)
+{
+    connect(this, SIGNAL(triggered()), this, SLOT(toggleLauncher()));
+    if (item->itemType() == TaskItemType) {
+        m_name = static_cast<TaskItem *>(item)->task()->classClass();
+    } else {
+        m_name = item->name();
+    }
+
+    if (item->itemType() == LauncherItemType) {
+        m_url = static_cast<LauncherItem *>(item)->url();
+        setText(i18n("Remove This Launcher"));
+    } else {
+        setText(i18n("&Show A Launcher For %1 When It Is Not Running", m_name));
+        setCheckable(true);
+
+        // Search for applications which are executable and case-insensitively match the windowclass of the task and
+        // See http://techbase.kde.org/Development/Tutorials/Services/Traders#The_KTrader_Query_Language
+        // if the following is unclear to you.
+        QString query = QString("exist Exec and ('%1' =~ Name)").arg(m_name);
+        KService::List services = KServiceTypeTrader::self()->query("Application", query);
+        if (!services.empty()) {
+            m_url = KUrl::fromPath((services[0]->entryPath()));
+        } else { // No desktop-file was found, so try to find at least the executable
+            QString path = KStandardDirs::findExe(m_name.toLower());
+            if (!path.isEmpty()) {
+                m_url = KUrl::fromPath(path);
+            }
+        }
+
+        if (m_url.isEmpty()) {
+            //don't show the possibility to add a launcher if we don't have a url for it
+            //kDebug() << "No executable found for" << m_name;
+            setVisible(false);
+            setChecked(false);
+        } else {
+            setChecked(m_groupingStrategy->launcherExists(m_url));
+        }
+    }
+}
+
+void ToggleLauncherActionImpl::toggleLauncher()
+{
+    if (!m_url.isValid()) {
+        return;
+    }
+
+    if (m_groupingStrategy->launcherExists(m_url)) {
+        m_groupingStrategy->removeLauncher(m_url);
+    } else if (m_url.isLocalFile() && KDesktopFile::isDesktopFile(m_url.toLocalFile())) {
+        m_groupingStrategy->addLauncher(m_url);
+    } else {
+        m_groupingStrategy->addLauncher(m_url, m_abstractItem->icon(), m_name);
+    }
+}
+
 EditGroupActionImpl::EditGroupActionImpl(QObject *parent, TaskGroup *group, GroupManager *groupManager)
     : QAction(parent)
 {
@@ -280,11 +383,15 @@ EditGroupActionImpl::EditGroupActionImpl(QObject *parent, TaskGroup *group, Grou
     connect(this, SIGNAL(triggered()), group, SIGNAL(groupEditRequest()));
     setText(i18n("&Edit Group"));
     //setIcon(KIcon("window-close"));
+    bool applicable = true;
     if (groupManager->groupingStrategy()) {
-        setEnabled(groupManager->taskGrouper()->editableGroupProperties());
+        applicable = groupManager->taskGrouper()->editableGroupProperties();
     } else {
-        setEnabled(false);
+        applicable = false;
     }
+
+    setEnabled(applicable);
+    setVisible(applicable);
 }
 
 GroupingStrategyMenu::GroupingStrategyMenu(QWidget *parent, AbstractGroupableItem* item, GroupManager *strategy)
@@ -303,7 +410,6 @@ GroupingStrategyMenu::GroupingStrategyMenu(QWidget *parent, AbstractGroupableIte
             }
         }
     }
-
 }
 
 
@@ -327,18 +433,7 @@ BasicMenu::BasicMenu(QWidget *parent, TaskItem* item, GroupManager *strategy, QL
     addAction(new MaximizeActionImpl(this, item));
     addAction(new ShadeActionImpl(this, item));
 
-    addMenu(new AdvancedMenu(this, item));
-
-    if (strategy->taskGrouper()) {
-        QList<QAction*> groupingStrategyActions = strategy->taskGrouper()->strategyActions(this, item);
-        if (!groupingStrategyActions.isEmpty()) {
-            addSeparator();
-            foreach (QAction *action, groupingStrategyActions) {
-                addAction(action);
-            }
-            // delete groupingStrategyActions;
-        }
-    }
+    addMenu(new AdvancedMenu(this, item, strategy));
 
     foreach (QAction *action, visualizationActions) {
         addAction(action);
@@ -357,12 +452,13 @@ BasicMenu::BasicMenu(QWidget *parent, TaskGroup* group, GroupManager *strategy,
     setTitle(group->name());
     setIcon(group->icon());
     foreach (AbstractGroupableItem *item, group->members()) {
-        if (item->isGroupItem()) {
+        if (item->itemType() == GroupItemType) {
             addMenu(new BasicMenu(this, dynamic_cast<TaskGroup*>(item), strategy));
         } else {
             addMenu(new BasicMenu(this, dynamic_cast<TaskItem*>(item), strategy));
         }
     }
+
     addSeparator();
 
     if (TaskManager::self()->numberOfDesktops() > 1) {
@@ -374,19 +470,10 @@ BasicMenu::BasicMenu(QWidget *parent, TaskGroup* group, GroupManager *strategy,
     addAction(new MaximizeActionImpl(this, group));
     addAction(new ShadeActionImpl(this, group));
 
-    addMenu(new AdvancedMenu(this, group));
-
-    if (strategy->taskGrouper()) {
-        QList<QAction*> groupingStrategyActions = strategy->taskGrouper()->strategyActions(this, group);
-        if (!groupingStrategyActions.isEmpty()) {
-            addSeparator();
-            foreach (QAction *action, groupingStrategyActions) {
-                addAction(action);
-            }
-        }
-    }
+    addMenu(new AdvancedMenu(this, group, strategy));
     addAction(new EditGroupActionImpl(this, group, strategy));
-    foreach(QAction *action, visualizationActions) {
+
+    foreach (QAction *action, visualizationActions) {
         addAction(action);
     }
 
@@ -394,6 +481,24 @@ BasicMenu::BasicMenu(QWidget *parent, TaskGroup* group, GroupManager *strategy,
     addAction(new CloseActionImpl(this, group));
 }
 
+BasicMenu::BasicMenu(QWidget *parent, LauncherItem* item, GroupManager *strategy, QList<QAction *> visualizationActions)
+    : QMenu(parent)
+{
+    Q_ASSERT(item);
+    Q_ASSERT(strategy);
+
+    setTitle(item->name());
+    setIcon(item->icon());
+
+    addAction(new ToggleLauncherActionImpl(this, item, strategy));
+
+    addSeparator();
+
+    foreach (QAction *action, visualizationActions) {
+        addAction(action);
+    }
+}
+
 GroupPopupMenu::GroupPopupMenu(QWidget *parent, TaskGroup *group, GroupManager *groupManager)
     :QMenu(parent)
 {
@@ -405,7 +510,7 @@ GroupPopupMenu::GroupPopupMenu(QWidget *parent, TaskGroup *group, GroupManager *
             continue;
         }
 
-        if (item->isGroupItem()) {
+        if (item->itemType() == GroupItemType) {
             QMenu* menu = new GroupPopupMenu (this, qobject_cast<TaskGroup*>(item), groupManager);
             addMenu(menu);
         } else {
