@@ -140,7 +140,6 @@ protected:
     void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
     void dragLeaveEvent(QGraphicsSceneDragDropEvent *event);
     void dropEvent(QGraphicsSceneDragDropEvent *event);
-    bool eventFilter(QObject *watched, QEvent *event);
     bool focusNextPrevChild(bool next);
 
     void handleDroppedId(WId id, AbstractTaskItem *targetTask, QGraphicsSceneDragDropEvent *event);
@@ -156,8 +155,8 @@ protected slots:
     virtual void wheelEvent(QGraphicsSceneWheelEvent *event);
 
 private Q_SLOTS:
+    void checkUpdates();
     void constraintsChanged(Plasma::Constraints);
-    void clearPopupLostFocus();
     void handleActiveWindowChanged(WId id);
 
     void updateTask(::TaskManager::TaskChanges changes);
@@ -174,6 +173,7 @@ private Q_SLOTS:
     void popupMenu();
     /** force a relayout of all items */
     void relayoutItems();
+    void popupVisibilityChanged(bool visible);
 
 private:
     AbstractTaskItem* createAbstractItem(AbstractGroupableItem * groupableItem);
@@ -211,7 +211,8 @@ private:
     bool m_collapsed;
     QGraphicsLinearLayout *m_mainLayout;
     Plasma::Dialog *m_popupDialog;
-    bool m_popupLostFocus;
+    QTimer *m_updateTimer;
+    TaskManager::TaskChanges m_changes;
 };
 
 #endif
