@@ -58,6 +58,7 @@ using TaskManager::AbstractGroupableItem;
 using TaskManager::GroupManager;
 using TaskManager::TaskItem;
 using TaskManager::TaskGroup;
+using TaskManager::LauncherItem;
 
 /**
  * An applet which provides a visual representation of running
@@ -104,12 +105,19 @@ public:
 
         void needsVisualFocus(bool focus);
         QWidget *popupDialog() const;
+
+        bool isPopupShowing() const;
+
 signals:
-    /**
-    * emitted whenever we receive a constraintsEvent
-    */
-    void constraintsChanged(Plasma::Constraints);
-    void settingsChanged();
+        /**
+         * emitted whenever we receive a constraintsEvent
+         */
+        void constraintsChanged(Plasma::Constraints);
+        void settingsChanged();
+
+public slots:
+        void configChanged();
+        void publishIconGeometry();
 
 protected slots:
         void configAccepted();
@@ -127,6 +135,8 @@ private slots:
         void reload();
         void changeSizeHint(Qt::SizeHint which);
         void dialogGroupingChanged(int index);
+        void launcherAdded(LauncherItem *launcher);
+        void launcherRemoved(LauncherItem *launcher);
 
 private:
         bool m_showTooltip;
