@@ -34,8 +34,7 @@
 #include <Plasma/Containment>
 #include <Plasma/Svg>
 
-#include <kephal/screens.h>
-
+#include "desktopcorona.h"
 #include "plasmaapp.h"
 #include "plasma-shell-desktop.h"
 #include "widgetsexplorer/widgetexplorer.h"
@@ -103,7 +102,7 @@ DashboardView::DashboardView(Plasma::Containment *containment, Plasma::View *vie
         setAttribute(Qt::WA_NoSystemBackground);
     }
 
-    setGeometry(Kephal::ScreenUtils::screenGeometry(containment->screen()));
+    setGeometry(PlasmaApp::self()->corona()->screenGeometry(containment->screen()));
 
     m_hideAction = new QAction(i18n("Hide Dashboard"), this);
     m_hideAction->setIcon(KIcon("preferences-desktop-display"));
@@ -134,7 +133,13 @@ void DashboardView::drawBackground(QPainter * painter, const QRectF & rect)
     if (PlasmaApp::hasComposite()) {
         setWallpaperEnabled(false);
         painter->setCompositionMode(QPainter::CompositionMode_Source);
-        painter->fillRect(rect, QColor(0, 0, 0, 180));
+ 
+	// KWin dahsboard plugin draws its own background for the dashboard
+        if (!Plasma::WindowEffects::isEffectAvailable(Plasma::WindowEffects::Dashboard)) {
+            painter->fillRect(rect, QColor(0, 0, 0, 180));
+        } else {
+            painter->fillRect(rect, QColor(0, 0, 0, 0));
+	}
     } else {
         setWallpaperEnabled(true);
         Plasma::View::drawBackground(painter, rect);
@@ -201,7 +206,7 @@ void DashboardView::showWidgetExplorer()
         m_widgetExplorer = widgetExplorer;
         widgetExplorer->installEventFilter(this);
         widgetExplorer->setContainment(c);
-        widgetExplorer->setOrientation(Qt::Horizontal);
+        widgetExplorer->setLocation(Plasma::BottomEdge);
         widgetExplorer->setIconSize(KIconLoader::SizeHuge);
         widgetExplorer->populateWidgetList();
         widgetExplorer->setMaximumWidth(width());
@@ -253,6 +258,9 @@ void DashboardView::showDashboard(bool showDashboard)
         setWindowFlags(Qt::FramelessWindowHint);
         setWindowState(Qt::WindowFullScreen);
 
+        // mark as dashboard
+        Plasma::WindowEffects::markAsDashboard(winId());
+
         if (AppSettings::perVirtualDesktopViews()) {
             //kDebug() << "pvdv dashboard, setting" << winId() << "on desktop" << m_view->desktop() + 1;
             KWindowSystem::setOnDesktop(winId(), m_view->desktop() + 1);
