@@ -1,5 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2008 by Marco Martin <notmart@gmail.com>                *
+ *   Copyright (C) 2010 by Chani Armitage <chanika@gmail.com>              *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -22,6 +23,7 @@
 #include <QGraphicsLinearLayout>
 #include <QGraphicsSceneMouseEvent>
 
+#include <KDebug>
 #include <KWindowSystem>
 #include <KIcon>
 
@@ -30,11 +32,11 @@
 #include <Plasma/Context>
 #include <Plasma/Containment>
 #include <Plasma/TabBar>
-
+#include <Plasma/ServiceJob>
 
 ActivityBar::ActivityBar(QObject *parent, const QVariantList &args)
     : Plasma::Applet(parent, args),
-      m_activeContainment(-1)
+      m_engine(0)
 {
     resize(200, 60);
     setAspectRatioMode(Plasma::IgnoreAspectRatio);
@@ -50,41 +52,56 @@ void ActivityBar::init()
 {
     QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(this);
     m_tabBar = new Plasma::TabBar(this);
-    m_tabBar->nativeWidget()->installEventFilter(this);
     layout->addItem(m_tabBar);
     layout->setContentsMargins(0,0,0,0);
     //layout->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
 
-    if (containment()) {
-        Plasma::Corona *c = containment()->corona();
-
-        if (!c) {
-            kDebug() << "No corona, can't happen";
-            setFailedToLaunch(true);
-            return;
+    //This is an awful hack, but I need to keep the old behaviour for plasma-netbook
+    //while using the new activity API for plasma-desktop.
+    //TODO 4.6 convert netbook to the activity API so we won't need this
+    if (qApp->applicationName() == "plasma-desktop") {
+        m_engine = dataEngine("org.kde.activities");
+        Plasma::DataEngine::Data data = m_engine->query("Status");
+        QStringList activities = data["Running"].toStringList();
+        //kDebug() << "$$$$$$$$$$$$# sources:" << activities.size();
+        foreach (const QString &id, activities) {
+            insertActivity(id);
         }
+        m_engine->connectAllSources(this);
+        connect(m_engine, SIGNAL(sourceAdded(QString)), this, SLOT(activityAdded(QString)));
+        connect(m_engine, SIGNAL(sourceRemoved(QString)), this, SLOT(activityRemoved(QString)));
+        connect(m_tabBar, SIGNAL(currentChanged(int)), this, SLOT(switchActivity(int)));
+    } else {
+        m_tabBar->nativeWidget()->installEventFilter(this);
+        if (containment()) {
+            Plasma::Corona *c = containment()->corona();
 
-        QList<Plasma::Containment*> containments = c->containments();
-        foreach (Plasma::Containment *cont, containments) {
-            if (cont->containmentType() == Plasma::Containment::PanelContainment || cont->containmentType() == Plasma::Containment::CustomPanelContainment || c->offscreenWidgets().contains(cont)) {
-                continue;
+            if (!c) {
+                kDebug() << "No corona, can't happen";
+                setFailedToLaunch(true);
+                return;
             }
 
-            insertContainment(cont);
+            QList<Plasma::Containment*> containments = c->containments();
+            foreach (Plasma::Containment *cont, containments) {
+                if (cont->containmentType() == Plasma::Containment::PanelContainment || cont->containmentType() == Plasma::Containment::CustomPanelContainment || c->offscreenWidgets().contains(cont)) {
+                    continue;
+                }
 
-            connect(cont, SIGNAL(destroyed(QObject *)), this, SLOT(containmentDestroyed(QObject *)));
-            connect(cont, SIGNAL(screenChanged(int, int, Plasma::Containment *)), this, SLOT(screenChanged(int, int, Plasma::Containment *)));
-            connect(cont, SIGNAL(contextChanged(Plasma::Context *)), this, SLOT(contextChanged(Plasma::Context *)));
+                insertContainment(cont);
+
+                connect(cont, SIGNAL(destroyed(QObject *)), this, SLOT(containmentDestroyed(QObject *)));
+                connect(cont, SIGNAL(screenChanged(int, int, Plasma::Containment *)), this, SLOT(screenChanged(int, int, Plasma::Containment *)));
+                connect(cont, SIGNAL(contextChanged(Plasma::Context *)), this, SLOT(contextChanged(Plasma::Context *)));
+            }
+
+            connect(c, SIGNAL(containmentAdded(Plasma::Containment *)), this, SLOT(containmentAdded(Plasma::Containment *)));
         }
 
-        connect(c, SIGNAL(containmentAdded(Plasma::Containment *)), this, SLOT(containmentAdded(Plasma::Containment *)));
-    }
-
-    if (m_containments.count() > 1) {
         connect(m_tabBar, SIGNAL(currentChanged(int)), this, SLOT(switchContainment(int)));
-    }
 
-    connect(KWindowSystem::self(), SIGNAL(currentDesktopChanged(int)), this, SLOT(currentDesktopChanged(int)));
+        connect(KWindowSystem::self(), SIGNAL(currentDesktopChanged(int)), this, SLOT(currentDesktopChanged(int)));
+    }
 
     setPreferredSize(m_tabBar->nativeWidget()->sizeHint());
     emit sizeHintChanged(Qt::PreferredSize);
@@ -122,11 +139,18 @@ void ActivityBar::insertContainment(Plasma::Containment *cont)
      if (cont->screen() != -1 &&
          cont->screen() == myScreen &&
          (cont->desktop() == -1 || cont->desktop() ==  KWindowSystem::currentDesktop()-1)) {
-         m_activeContainment = index;
-         m_tabBar->setCurrentIndex(m_activeContainment);
+         m_tabBar->setCurrentIndex(index);
      }
 }
 
+void ActivityBar::insertActivity(const QString &id)
+{
+    //assumption: activities are always added at the end of the list
+    //kDebug() << "activity" << id;
+    m_activities.append(id);
+    m_tabBar->addTab(QString()); //name will be added on dataUpdated
+}
+
 void ActivityBar::constraintsEvent(Plasma::Constraints constraints)
 {
     if (constraints & Plasma::FormFactorConstraint ) {
@@ -135,18 +159,13 @@ void ActivityBar::constraintsEvent(Plasma::Constraints constraints)
         } else {
             m_tabBar->nativeWidget()->setShape(QTabBar::RoundedNorth);
         }
+
+        m_tabBar->nativeWidget()->setDrawBase(formFactor() != Plasma::Vertical && formFactor() != Plasma::Horizontal);
+
         setPreferredSize(m_tabBar->nativeWidget()->sizeHint());
         setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
         emit sizeHintChanged(Qt::PreferredSize);
     }
-
-    if (constraints & Plasma::SizeConstraint ) {
-        Plasma::Containment *c = containment();
-        if (c) {
-            const bool drawBase = size().width() + 2 <= c->size().width() && size().height() + 2 <= c->size().height();
-            m_tabBar->nativeWidget()->setDrawBase(drawBase);
-        }
-    }
 }
 
 bool ActivityBar::eventFilter(QObject *watched, QEvent *event)
@@ -173,7 +192,6 @@ void ActivityBar::switchContainment(int newActive)
 
     Plasma::Containment *oldCont = c->containmentForScreen(ownCont->screen(), KWindowSystem::currentDesktop() - 1);
 
-    m_activeContainment = newActive;
     if (oldCont) {
         m_containments[newActive]->setScreen(oldCont->screen(), oldCont->desktop());
     } else {
@@ -181,6 +199,17 @@ void ActivityBar::switchContainment(int newActive)
     }
 }
 
+void ActivityBar::switchActivity(int newActive)
+{
+    if (newActive >= m_activities.count() || newActive < 0) {
+        return;
+    }
+    Plasma::Service *service = m_engine->serviceForSource(m_activities.at(newActive));
+    KConfigGroup op = service->operationDescription("setCurrent");
+    Plasma::ServiceJob *job = service->startOperationCall(op);
+    connect(job, SIGNAL(finished(KJob*)), service, SLOT(deleteLater()));
+}
+
 void ActivityBar::currentDesktopChanged(const int currentDesktop)
 {
     Plasma::Corona *c = containment()->corona();
@@ -197,9 +226,7 @@ void ActivityBar::currentDesktopChanged(const int currentDesktop)
 
     int index = m_containments.indexOf(cont);
 
-    if (index != -1 &&
-        index != m_activeContainment) {
-        m_activeContainment = index;
+    if (index != -1) {
         m_tabBar->setCurrentIndex(index);
     }
 }
@@ -214,10 +241,6 @@ void ActivityBar::containmentAdded(Plasma::Containment *cont)
 
     insertContainment(cont);
 
-    if (m_containments.count() > 1) {
-        connect(m_tabBar, SIGNAL(currentChanged(int)), this, SLOT(switchContainment(int)));
-    }
-
     connect(cont, SIGNAL(destroyed(QObject *)), this, SLOT(containmentDestroyed(QObject *)));
     connect(cont, SIGNAL(screenChanged(int, int, Plasma::Containment *)), this, SLOT(screenChanged(int, int, Plasma::Containment *)));
     connect(cont, SIGNAL(contextChanged(Plasma::Context *)), this, SLOT(contextChanged(Plasma::Context *)));
@@ -226,20 +249,21 @@ void ActivityBar::containmentAdded(Plasma::Containment *cont)
     emit sizeHintChanged(Qt::PreferredSize);
 }
 
+void ActivityBar::activityAdded(const QString &id)
+{
+    insertActivity(id);
+    m_engine->connectSource(id, this);
+
+    setPreferredSize(m_tabBar->nativeWidget()->sizeHint());
+    emit sizeHintChanged(Qt::PreferredSize);
+}
+
 void ActivityBar::containmentDestroyed(QObject *obj)
 {
     Plasma::Containment *containment = static_cast<Plasma::Containment *>(obj);
 
     int index = m_containments.indexOf(containment);
     if (index != -1) {
-        if (index < m_activeContainment) {
-            --m_activeContainment;
-        }
-
-        if (m_containments.count() == 1) {
-            disconnect(m_tabBar, SIGNAL(currentChanged(int)), this, SLOT(switchContainment(int)));
-        }
-
         m_containments.removeAt(index);
         m_tabBar->blockSignals(true);
         m_tabBar->removeTab(index);
@@ -250,6 +274,23 @@ void ActivityBar::containmentDestroyed(QObject *obj)
     emit sizeHintChanged(Qt::PreferredSize);
 }
 
+void ActivityBar::activityRemoved(const QString &id)
+{
+    int index = m_activities.indexOf(id);
+    if (index < 0) {
+        return;
+    }
+
+    m_activities.removeAt(index);
+
+    m_tabBar->blockSignals(true);
+    m_tabBar->removeTab(index);
+    m_tabBar->blockSignals(false);
+
+    setPreferredSize(m_tabBar->nativeWidget()->sizeHint());
+    emit sizeHintChanged(Qt::PreferredSize);
+}
+
 void ActivityBar::screenChanged(int wasScreen, int isScreen, Plasma::Containment *cont)
 {
     Q_UNUSED(wasScreen)
@@ -259,10 +300,8 @@ void ActivityBar::screenChanged(int wasScreen, int isScreen, Plasma::Containment
 
     //FIXME: how is supposed to work containment()->desktop() when the pervirtialthing is off?
     if (index != -1 &&
-        index != m_activeContainment &&
         containment()->screen() == isScreen &&
         (cont->desktop() == -1 || cont->desktop() == KWindowSystem::currentDesktop()-1)) {
-        m_activeContainment = index;
         m_tabBar->setCurrentIndex(index);
     }
 }
@@ -281,4 +320,42 @@ void ActivityBar::contextChanged(Plasma::Context *context)
     }
 }
 
+void ActivityBar::dataUpdated(const QString &source, const Plasma::DataEngine::Data &data)
+{
+    //kDebug() << "$$$$$$$$$$$$$$$$$$$" << source;
+    if (source == "Status") {
+        //special source, not used yet
+        return;
+    }
+
+    int index = m_activities.indexOf(source);
+    if (data["State"].toString() == "Stopped") {
+        if (index >= 0) {
+            //take it out
+            m_activities.removeAt(index);
+            m_tabBar->blockSignals(true);
+            m_tabBar->removeTab(index);
+            m_tabBar->blockSignals(false);
+
+            setPreferredSize(m_tabBar->nativeWidget()->sizeHint());
+            emit sizeHintChanged(Qt::PreferredSize);
+        }
+        return;
+    } else if (index < 0) {
+        //add it back in
+        index = m_activities.size();
+        insertActivity(source);
+    }
+
+    //update the actual content
+    m_tabBar->setTabText(index, data["Name"].toString());
+    m_tabBar->setTabIcon(index, KIcon(data["Icon"].toString()));
+    if (data["Current"].toBool()) {
+        m_tabBar->setCurrentIndex(index);
+    }
+
+    setPreferredSize(m_tabBar->nativeWidget()->sizeHint());
+    emit sizeHintChanged(Qt::PreferredSize);
+}
+
 #include "activitybar.moc"
