@@ -27,6 +27,7 @@
 
 #include <KDebug>
 #include <KDesktopFile>
+#include <KGlobalSettings>
 #include <KIconLoader>
 #include <KLocale>
 #include <KMenu>
@@ -34,6 +35,7 @@
 #include <KRun>
 #include <KSharedConfig>
 #include <KShell>
+#include <KSycoca>
 #include <KUrl>
 #include <KWindowSystem>
 #include <kio/copyjob.h>
@@ -84,6 +86,9 @@ void IconApplet::init()
     setDisplayLines(2);
     registerAsDragHandle(m_icon);
     setAspectRatioMode(Plasma::ConstrainedSquare);
+
+    connect(KGlobalSettings::self(), SIGNAL(iconChanged(int)),
+        this, SLOT(iconSizeChanged(int)));
 }
 
 IconApplet::~IconApplet()
@@ -110,11 +115,30 @@ void IconApplet::saveState(KConfigGroup &cg) const
     }
 }
 
+void IconApplet::checkService(const QStringList &changedResources)
+{
+    if (changedResources.contains("apps")) {
+        setUrl(m_url);
+    }
+}
+
+void IconApplet::iconSizeChanged(int group)
+{
+    if (group == KIconLoader::Desktop || group == KIconLoader::Panel) {
+        updateGeometry();
+    }
+}
+
 void IconApplet::setUrl(const KUrl& url)
 {
-    m_url = KIO::NetAccess::mostLocalUrl(url, 0);
+    m_url = url;
+    if (!m_url.protocol().isEmpty()) {
+        m_url = KIO::NetAccess::mostLocalUrl(url, 0);
+    }
 
-    m_mimetype = KMimeType::findByUrl(url);
+    m_service = 0;
+    disconnect(KSycoca::self(), SIGNAL(databaseChanged(QStringList)),
+               this, SLOT(checkService(QStringList)));
 
     delete m_watcher;
     m_watcher = 0;
@@ -138,19 +162,27 @@ void IconApplet::setUrl(const KUrl& url)
         connect(m_watcher, SIGNAL(dirty(const QString &)), this, SLOT(updateDesktopFile()));
     } else {
         m_text = m_url.fileName();
+        m_service = KService::serviceByStorageId(m_text);
+        connect(KSycoca::self(), SIGNAL(databaseChanged(QStringList)),
+                this, SLOT(checkService(QStringList)));
 
-        if (m_text.isEmpty() && m_url.isLocalFile()) {
-            //handle special case like the / folder
-            m_text = m_url.directory();
-        } else if(m_text.isEmpty()) {
-            m_text = m_url.prettyUrl();
+        if (m_service) {
+            m_text = m_service->name();
+            m_icon->setIcon(m_service->icon());
+        } else {
+            if (m_text.isEmpty() && m_url.isLocalFile()) {
+                //handle special case like the / folder
+                m_text = m_url.directory();
+            } else if (m_text.isEmpty()) {
+                m_text = m_url.prettyUrl();
 
-            if(m_text.endsWith(":/")) {
-                m_text = m_url.protocol();
+                if (m_text.endsWith(QLatin1String(":/"))) {
+                    m_text = m_url.protocol();
+                }
             }
-        }
 
-        m_icon->setIcon(KMimeType::iconNameForUrl(url));
+            m_icon->setIcon(KMimeType::iconNameForUrl(url));
+        }
     }
 
     if (m_icon->icon().isNull()) {
@@ -188,7 +220,11 @@ void IconApplet::updateDesktopFile()
 
 void IconApplet::openUrl()
 {
-    if (m_url.isValid()) {
+    if (m_service) {
+        emit releaseVisualFocus();
+        KUrl::List urls;
+        KRun::run(*m_service, urls, 0);
+    } else if (m_url.isValid()) {
         emit releaseVisualFocus();
         new KRun(m_url, 0);
     }
@@ -252,6 +288,29 @@ void IconApplet::showConfigurationInterface()
     }
 }
 
+QSizeF IconApplet::sizeHint(Qt::SizeHint which, const QSizeF & constraint) const
+{
+    if (which == Qt::PreferredSize) {
+        int iconSize;
+
+        switch (formFactor()) {
+            case Plasma::Planar:
+            case Plasma::MediaCenter:
+                iconSize = IconSize(KIconLoader::Desktop);
+                break;
+
+            case Plasma::Horizontal:
+            case Plasma::Vertical:
+                iconSize = IconSize(KIconLoader::Panel);
+                break;
+        }
+
+        return QSizeF(iconSize, iconSize);
+    }
+
+    return Plasma::Applet::sizeHint(which, constraint);
+}
+
 void IconApplet::setDisplayLines(int displayLines)
 {
     if (m_icon) {
@@ -292,23 +351,28 @@ void IconApplet::dropEvent(QGraphicsSceneDragDropEvent *event)
     }
 
     KUrl::List urls = KUrl::List::fromMimeData(event->mimeData());
-
-    if (!urls.isEmpty()) {
-        event->accept();
-    } else {
+    if (urls.isEmpty()) {
         return;
     }
 
+    event->accept();
 
     if (m_url.isEmpty()) {
         setUrl(urls.first());
         //TODO: why we don't call updateConstraints()?
         constraintsEvent(Plasma::FormFactorConstraint);
-    } else if (m_url.isLocalFile() &&
-               m_mimetype &&
-               (m_mimetype->is("application/x-executable") ||
-                m_mimetype->is("application/x-shellscript") ||
-                KDesktopFile::isDesktopFile(m_url.toLocalFile()))) {
+        return;
+    } else if (m_service) {
+        KRun::run(*m_service, urls, 0);
+        return;
+    }
+
+    KMimeType::Ptr mimetype = KMimeType::findByUrl(m_url);
+
+    if (m_url.isLocalFile() &&
+        ((mimetype && (mimetype->is("application/x-executable") ||
+                       mimetype->is("application/x-shellscript"))) ||
+          KDesktopFile::isDesktopFile(m_url.toLocalFile()))) {
 
         //Parameters
         QString params;
@@ -324,33 +388,16 @@ void IconApplet::dropEvent(QGraphicsSceneDragDropEvent *event)
         QString commandStr;
         //Extract the command from the Desktop file
         if (KDesktopFile::isDesktopFile(m_url.toLocalFile())) {
-            KDesktopFile f(m_url.toLocalFile());
-            KConfigGroup config = f.desktopGroup();
-            commandStr = config.readPathEntry( "Exec", QString() );
-
-            if (commandStr.isEmpty()) {
-                QString path = f.readUrl();
-                if (path.isEmpty()) {
-                    path = f.readPath();
-                }
-
-                if (path.isEmpty()) {
-                    return;
-                }
-
-                KUrl dest(path);
-                KMimeType::Ptr mime = KMimeType::findByUrl(dest);
-                if (m_mimetype->is("inode/directory")) {
-                    dropUrls(urls, dest, event->modifiers());
-                }
-            }
+            KService service(m_url.toLocalFile());
+            KRun::run(service, urls, 0);
+            return;
         } else {
             //Else just exec the local executable
             commandStr = KShell::quoteArg(m_url.path());
         }
 
         KRun::runCommand(commandStr + ' ' + params, 0);
-    } else if (m_mimetype->is("inode/directory")) {
+    } else if (mimetype && mimetype->is("inode/directory")) {
         dropUrls(urls, m_url, event->modifiers());
     }
 }
