@@ -24,6 +24,7 @@
 #include "plasmoidtask.h"
 #include <fixx11h.h>
 
+#include <KDebug>
 #include <KIcon>
 #include <KIconLoader>
 
@@ -35,156 +36,143 @@
 namespace SystemTray
 {
 
-class PlasmoidTask::Private
-{
-public:
-    Private(QString name, int appletId, PlasmoidTask *q, Plasma::Applet *parentApplet)
-        : q(q),
-          name(name),
-          id(appletId),
-          typeId(name),
-          applet(0),
-          host(parentApplet),
-          takenByParent(false)
-    {
-        if (!name.isEmpty()) {
-            setupApplet();
-        }
-    }
-
-    void setupApplet();
-
-    PlasmoidTask *q;
-    QString name;
-    int id;
-    QString typeId;
-    QIcon icon;
-    Plasma::Applet *applet;
-    Plasma::Applet *host;
-    bool takenByParent;
-};
-
-
-PlasmoidTask::PlasmoidTask(QString appletname, int id, QObject *parent, Plasma::Applet *host)
+PlasmoidTask::PlasmoidTask(const QString &appletname, int id, QObject *parent, Plasma::Applet *host)
     : Task(parent),
-      d(new Private(appletname, id, this, host))
+      m_name(appletname),
+      m_typeId(appletname),
+      m_host(host),
+      m_takenByParent(false)
 {
+    setupApplet(appletname, id);
 }
 
 
 PlasmoidTask::~PlasmoidTask()
 {
-    emit taskDeleted(d->host, d->typeId);
-    delete d;
+    emit taskDeleted(m_host, m_typeId);
 }
 
 
 bool PlasmoidTask::isEmbeddable() const
 {
-    return d->applet != 0 && !d->takenByParent;
+    return m_applet && !m_takenByParent;
 }
 
 bool PlasmoidTask::isValid() const
 {
-    return !d->name.isEmpty();
+    return !m_name.isEmpty() && m_applet;
 }
 
 QString PlasmoidTask::name() const
 {
-    return d->name;
+    if (m_applet) {
+        return m_applet.data()->name();
+    }
+
+    return m_name;
 }
 
 
 QString PlasmoidTask::typeId() const
 {
-    return d->typeId;
+    return m_typeId;
 }
 
 
 QIcon PlasmoidTask::icon() const
 {
-    return d->icon;
+    return m_icon;
 }
 
 Plasma::Applet *PlasmoidTask::host() const
 {
-    return d->host;
+    return m_host;
 }
 
 QGraphicsWidget* PlasmoidTask::createWidget(Plasma::Applet *host)
 {
-    if (host != d->host || !d->applet) {
+    if (host != m_host || !m_applet) {
         return 0;
     }
 
-    d->takenByParent = true;
-    d->applet->setParent(host);
-    d->applet->setParentItem(host);
-    d->applet->init();
-    d->applet->updateConstraints(Plasma::StartupCompletedConstraint);
-    d->applet->flushPendingConstraintsEvents();
-    d->applet->updateConstraints(Plasma::AllConstraints);
-    d->applet->flushPendingConstraintsEvents();
+    Plasma::Applet *applet = m_applet.data();
+    m_takenByParent = true;
+    applet->setParent(host);
+    applet->setParentItem(host);
+    applet->init();
+    applet->updateConstraints(Plasma::StartupCompletedConstraint);
+    applet->flushPendingConstraintsEvents();
+    applet->updateConstraints(Plasma::AllConstraints);
+    applet->flushPendingConstraintsEvents();
 
     // make sure to record it in the configuration so that if we reload from the config,
     // this applet is remembered
     KConfigGroup dummy;
-    d->applet->save(dummy);
+    applet->save(dummy);
 
-    connect(d->applet, SIGNAL(newStatus(Plasma::ItemStatus)), this, SLOT(newAppletStatus(Plasma::ItemStatus)));
+    connect(applet, SIGNAL(newStatus(Plasma::ItemStatus)), this, SLOT(newAppletStatus(Plasma::ItemStatus)));
 
-    newAppletStatus(d->applet->status());
+    newAppletStatus(applet->status());
 
-    connect(d->applet, SIGNAL(configNeedsSaving()), host, SIGNAL(configNeedsSaving()));
-    connect(d->applet, SIGNAL(releaseVisualFocus()), host, SIGNAL(releaseVisualFocus()));
+    connect(applet, SIGNAL(configNeedsSaving()), host, SIGNAL(configNeedsSaving()));
+    connect(applet, SIGNAL(releaseVisualFocus()), host, SIGNAL(releaseVisualFocus()));
 
-    return static_cast<QGraphicsWidget*>(d->applet);
+    return static_cast<QGraphicsWidget*>(applet);
 }
 
 void PlasmoidTask::forwardConstraintsEvent(Plasma::Constraints constraints)
 {
-    if (d->applet) {
-        d->applet->updateConstraints(constraints);
-        d->applet->flushPendingConstraintsEvents();
+    Plasma::Applet *applet = m_applet.data();
+    if (applet) {
+        applet->updateConstraints(constraints);
+        applet->flushPendingConstraintsEvents();
     }
 }
 
-void PlasmoidTask::Private::setupApplet()
+void PlasmoidTask::setupApplet(const QString &plugin, int id)
 {
-    applet = Plasma::Applet::load(name, id);
+    Plasma::Applet *applet = Plasma::Applet::load(plugin, id);
+    m_applet = applet;
 
-    if (!applet) {
-        kDebug() << "Could not load applet" << name;
-        name.clear();
+    if (!m_applet) {
+        kDebug() << "Could not load applet" << plugin;
         return;
     }
 
-    icon = KIcon(applet->icon());
+    //FIXME: System Information should be system services, but battery and devicenotifier are both there. we would need multiple categories
+    if (applet->category() == "System Information" ||
+        applet->category() == "Network") {
+        setCategory(Hardware);
+    } else if (applet->category() == "Online Services") {
+        setCategory(Communications);
+    }
+
+    m_icon = KIcon(applet->icon());
 
-    //applet->setParent(q);
     applet->setFlag(QGraphicsItem::ItemIsMovable, false);
 
-    connect(applet, SIGNAL(destroyed(QObject*)), q, SLOT(appletDestroyed(QObject*)));
+    connect(applet, SIGNAL(destroyed(QObject*)), this, SLOT(appletDestroyed(QObject*)));
     applet->setBackgroundHints(Plasma::Applet::NoBackground);
 
-
     applet->setPreferredSize(KIconLoader::SizeSmallMedium+2, KIconLoader::SizeSmallMedium+2);
     kDebug() << applet->name() << " Applet loaded";
 }
 
-void PlasmoidTask::appletDestroyed(QObject *object)
+void PlasmoidTask::appletDestroyed(QObject *)
 {
-    if (object == d->applet) {
-        d->applet = 0;
-        deleteLater();
-    }
+    deleteLater();
 }
 
 void PlasmoidTask::newAppletStatus(Plasma::ItemStatus status)
 {
+    Plasma::Applet *applet = m_applet.data();
+    if (!applet) {
+        return;
+    }
+
     switch (status) {
     case Plasma::PassiveStatus:
-       if (Plasma::PopupApplet *popupApplet = qobject_cast<Plasma::PopupApplet *>(d->applet)) {
+       if (Plasma::PopupApplet *popupApplet = qobject_cast<Plasma::PopupApplet *>(applet)) {
            popupApplet->hidePopup();
        }
        setStatus(Task::Passive);
