@@ -29,6 +29,7 @@
 #include <KIcon>
 #include <KBookmarkManager>
 #include <KMimeType>
+#include <KMimeTypeTrader>
 #include <KToolInvocation>
 #include <KUrl>
 #include <KStandardDirs>
@@ -39,7 +40,7 @@ BookmarksRunner::BookmarksRunner( QObject* parent, const QVariantList &args )
     : Plasma::AbstractRunner(parent, args)
 {
     Q_UNUSED(args)
-    setObjectName("Bookmarks");
+    setObjectName( QLatin1String("Bookmarks" ));
     m_icon = KIcon("bookmarks");
     m_bookmarkManager = KBookmarkManager::userBookmarksManager();
     m_browser = whichBrowser();
@@ -70,7 +71,8 @@ void BookmarksRunner::reloadConfiguration()
             KConfigGroup grp = config();
             /* This allows the user to specify a profile database */
             m_dbFile = grp.readEntry<QString>("dbfile", "");
-            if (m_dbFile.isEmpty()) {//Try to get the right database file, the default profile is used
+            if (m_dbFile.isEmpty() || QFile::exists(m_dbFile)) {
+                //Try to get the right database file, the default profile is used
                 KConfig firefoxProfile(QDir::homePath() + "/.mozilla/firefox/profiles.ini",
                                        KConfig::SimpleConfig);
                 QStringList profilesList = firefoxProfile.groupList();
@@ -121,6 +123,8 @@ void BookmarksRunner::prep()
                 m_dbCacheFile = KStandardDirs::locateLocal("cache", "") + "bookmarkrunnerfirefoxdbfile.sqlite";
             }
 
+            // ### DO NOT USE KIO FROM RUNNER THREADS!
+            // ### This looks like a local copy, so use QFile::copy instead.
             KIO::Job *job = KIO::file_copy(m_dbFile, m_dbCacheFile, -1,
                                            KIO::HideProgressInfo | KIO::Overwrite);
             connect(job, SIGNAL(result(KJob*)), this, SLOT(dbCopied(KJob*)));
@@ -301,6 +305,11 @@ void BookmarksRunner::matchKonquerorBookmarks(Plasma::RunnerContext& context, bo
             return;
         }
 
+        if (bookmark.isSeparator()) {
+            bookmark = bookmarkGroup.next(bookmark);
+            continue;
+        }
+
         if (bookmark.isGroup()) { // descend
             //kDebug () << "descending into" << bookmark.text();
             groups.push(bookmarkGroup);
@@ -378,10 +387,7 @@ void BookmarksRunner::matchFirefoxBookmarks(Plasma::RunnerContext& context, bool
         return;
     }
 
-    QList<int> fks;
-    QHash<int, QString> titles; //index (fk), title
-    QHash<int, QUrl> urls; //index, url (QUrl in order to go with QVariant)
-    QHash<int, KIcon> icons; //index, icon
+    QList<Plasma::QueryMatch> matches;
     QString tmpTerm = term;
     QSqlQuery query;
     if (allBookmarks) {
@@ -396,28 +402,46 @@ void BookmarksRunner::matchFirefoxBookmarks(Plasma::RunnerContext& context, bool
                         "(moz_bookmarks.title LIKE  '%" + escapedTerm + "%' or moz_places.url LIKE '%"
                         + escapedTerm + "%')");
     }
-    while (query.next()) {
+
+    while (query.next() && context.isValid()) {
         const QString title = query.value(1).toString();
         const QUrl url = query.value(2).toString();
         //const int favicon_id = query.value(3).toInt();
 
-        int fk = query.value(0).toInt();
-        fks << fk;
-        titles.insert(fk, title);
-        urls.insert(fk, url);
-        icons.insert(fk, m_icon); //could be changed to use favicon
-    }
+        if (title.isEmpty() || url.isEmpty() || url.scheme().contains("place")) {
+            //Don't use bookmarks with empty title, url or Firefox intern url
+            kDebug() << "element was not added";
+            continue;
+        }
 
-    if (!context.isValid()) {
-        return;
-    }
+        Plasma::QueryMatch::Type type = Plasma::QueryMatch::NoMatch;
+        qreal relevance = 0;
+
+        if (title.compare(term, Qt::CaseInsensitive) == 0) {
+            type = Plasma::QueryMatch::ExactMatch;
+            relevance = 1.0;
+        } else if (title.contains(term, Qt::CaseInsensitive)) {
+            type = Plasma::QueryMatch::PossibleMatch;
+            relevance = 0.45;
+        } else if (url.toString().contains(term, Qt::CaseInsensitive)) {
+            type = Plasma::QueryMatch::PossibleMatch;
+            relevance = 0.2;
+        } else if (allBookmarks) {
+            type = Plasma::QueryMatch::PossibleMatch;
+            relevance = 0.18;
+        }
 
-    QList<Plasma::QueryMatch> matches;
-    foreach (const int& fk, fks) {
         Plasma::QueryMatch match(this);
-        match.setIcon(icons[fk]);
-        match.setText(titles[fk]);
-        match.setData(urls[fk]);
+        QIcon icon = favicon(url);
+        if (icon.isNull()) {
+            match.setIcon(m_icon);
+        } else {
+            match.setIcon(icon);
+        }
+        match.setText(title);
+        match.setData(url);
+        match.setType(type);
+        match.setRelevance(relevance);
         matches << match;
     }
     context.addMatches(term, matches);
@@ -425,8 +449,16 @@ void BookmarksRunner::matchFirefoxBookmarks(Plasma::RunnerContext& context, bool
 
 BookmarksRunner::Browser BookmarksRunner::whichBrowser()
 {
+    //HACK find the default browser
     KConfigGroup config(KSharedConfig::openConfig("kdeglobals"), QLatin1String("General") );
-    QString exec = config.readPathEntry( QLatin1String("BrowserApplication"), QString("") );
+    QString exec = config.readPathEntry(QLatin1String("BrowserApplication"), QString());
+    if (exec.isEmpty()) {
+        KService::Ptr service = KMimeTypeTrader::self()->preferredService("text/html");
+        if (service) {
+            exec = service->exec();
+        }
+    }
+
     //kDebug() << exec;
     if (exec.contains("firefox", Qt::CaseInsensitive)) {
         return Firefox;
@@ -440,7 +472,29 @@ BookmarksRunner::Browser BookmarksRunner::whichBrowser()
 void BookmarksRunner::run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &action)
 {
     Q_UNUSED(context);
-    KUrl url = (KUrl)action.data().toString();
+    const QString term = action.data().toString();
+    KUrl url = KUrl(term);
+
+    //support urls like "kde.org" by transforming them to http://kde.org
+    if (url.protocol().isEmpty()) {
+        const int idx = term.indexOf('/');
+
+        url.clear();
+        url.setHost(term.left(idx));
+        if (idx != -1) {
+            //allow queries
+            const int queryStart = term.indexOf('?', idx);
+            int pathLength = -1;
+            if ((queryStart > -1) && (idx < queryStart)) {
+                pathLength = queryStart - idx;
+                url.setQuery(term.mid(queryStart));
+            }
+
+            url.setPath(term.mid(idx, pathLength));
+        }
+        url.setProtocol("http");
+    }
+
     //kDebug() << "BookmarksRunner::run opening: " << url.url();
     KToolInvocation::invokeBrowser(url.url());
 }
