@@ -1,6 +1,5 @@
 /*  This file is part of the KDE project
     Copyright (C) 2006 Will Stephenson <wstephenson@kde.org>
-    Copyright (C) 2008 Pino Toscano <pino@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -18,7 +17,7 @@
 
 */
 
-#include "NetworkManager-networkmanager.h"
+#include "manager.h"
 
 // KDE includes
 #include <kpluginfactory.h>
