@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2009 by Ben Cooksley <ben@eclipse.endoftheinternet.org> *
+ *   Copyright (C) 2009 by Ben Cooksley <bcooksley@kde.org>                *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -90,7 +90,7 @@ void SettingsBase::initApplication()
             const KService::Ptr entry = modules.at(i);
             MenuItem * infoItem = new MenuItem(false, lostFound);
             infoItem->setService( entry );
-            kWarning() << "Added " << entry->name();
+            kDebug() << "Added " << entry->name();
         }
     }
 
@@ -194,7 +194,10 @@ void SettingsBase::initMenuList(MenuItem * parent)
     for (int i = 0; i < categories.size(); ++i) {
         const KService::Ptr entry = categories.at(i);
         const QString parentCategory = entry->property("X-KDE-System-Settings-Parent-Category").toString();
-        if ( parentCategory == parent->category() ) {
+        const QString parentCategory2 = entry->property("X-KDE-System-Settings-Parent-Category-V2").toString();
+        if ( parentCategory == parent->category() ||
+             // V2 entries must not be empty if they want to become a proper category.
+             ( !parentCategory2.isEmpty() && parentCategory2 == parent->category() ) ) {
             MenuItem * menuItem = new MenuItem(true, parent);
             menuItem->setService( entry );
             if( menuItem->category() == "lost-and-found" ) {
